# -*- coding: utf-8 -*-
"""This file contains a parser for the Kik database on iOS.

Kik messages on iOS devices are stored in an
SQLite database file named kik.sqlite.
"""

from dfdatetime import cocoa_time as dfdatetime_cocoa_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class KikIOSMessageEventData(events.EventData):
  """Kik message event data.

  Args:
    body (str): content of the message.
    message_status (str): message status, such as:
        read, unread, not sent, delivered, etc.
    message_type (str): message type, either Sent or Received.
    username (str): unique username of the sender or receiver.
  """

  DATA_TYPE = u'ios:kik:messaging'

  def __init__(self):
    """Initializes event data."""
    super(KikIOSMessageEventData, self).__init__(data_type=self.DATA_TYPE)
    self.body = None
    self.displayname = None
    self.message_status = None
    self.message_type = None
    self.username = None


class KikIOSPlugin(interface.SQLitePlugin):
  """SQLite plugin for Kik iOS database."""

  NAME = u'kik_messenger'
  DESCRIPTION = u'Parser for iOS Kik messenger SQLite database files.'

  # Define the needed queries.
  QUERIES = [
      (u'SELECT a.Z_PK AS id, b.ZUSERNAME, b.ZDISPLAYNAME,'
       u'a.ZRECEIVEDTIMESTAMP, a.ZSTATE, a.ZTYPE, a.ZBODY '
       u'FROM ZKIKMESSAGE a JOIN ZKIKUSER b ON b.ZEXTRA = a.ZUSER',
       u'ParseMessageRow')]

  # The required tables.
  REQUIRED_TABLES = frozenset([u'ZKIKMESSAGE', u'ZKIKUSER'])

  SCHEMAS = [{
      u'Z_3MESSAGES': (
          u'CREATE TABLE Z_3MESSAGES ( Z_3CHAT INTEGER, Z_5MESSAGES INTEGER, '
          u'PRIMARY KEY (Z_3CHAT, Z_5MESSAGES) )'),
      u'Z_6ADMINSINVERSE': (
          u'CREATE TABLE Z_6ADMINSINVERSE ( Z_6ADMINS INTEGER, '
          u'Z_6ADMINSINVERSE INTEGER, PRIMARY KEY (Z_6ADMINS, '
          u'Z_6ADMINSINVERSE) )'),
      u'Z_6BANSINVERSE': (
          u'CREATE TABLE Z_6BANSINVERSE ( Z_6BANS INTEGER, Z_6BANSINVERSE '
          u'INTEGER, PRIMARY KEY (Z_6BANS, Z_6BANSINVERSE) )'),
      u'Z_6MEMBERS': (
          u'CREATE TABLE Z_6MEMBERS ( Z_6MEMBERSINVERSE INTEGER, Z_6MEMBERS '
          u'INTEGER, PRIMARY KEY (Z_6MEMBERSINVERSE, Z_6MEMBERS) )'),
      u'Z_METADATA': (
          u'CREATE TABLE Z_METADATA (Z_VERSION INTEGER PRIMARY KEY, Z_UUID '
          u'VARCHAR(255), Z_PLIST BLOB)'),
      u'Z_PRIMARYKEY': (
          u'CREATE TABLE Z_PRIMARYKEY (Z_ENT INTEGER PRIMARY KEY, Z_NAME '
          u'VARCHAR, Z_SUPER INTEGER, Z_MAX INTEGER)'),
      u'ZKIKATTACHMENT': (
          u'CREATE TABLE ZKIKATTACHMENT ( Z_PK INTEGER PRIMARY KEY, Z_ENT '
          u'INTEGER, Z_OPT INTEGER, ZFLAGS INTEGER, ZINTERNALID INTEGER, '
          u'ZRETRYCOUNT INTEGER, ZSTATE INTEGER, ZTYPE INTEGER, ZEXTRA '
          u'INTEGER, ZMESSAGE INTEGER, ZLASTACCESSTIMESTAMP TIMESTAMP, '
          u'ZTIMESTAMP TIMESTAMP, ZCONTENT VARCHAR )'),
      u'ZKIKATTACHMENTEXTRA': (
          u'CREATE TABLE ZKIKATTACHMENTEXTRA ( Z_PK INTEGER PRIMARY KEY, Z_ENT '
          u'INTEGER, Z_OPT INTEGER, ZATTACHMENT INTEGER, ZENCRYPTIONKEY '
          u'BLOB )'),
      u'ZKIKCHAT': (
          u'CREATE TABLE ZKIKCHAT ( Z_PK INTEGER PRIMARY KEY, Z_ENT INTEGER, '
          u'Z_OPT INTEGER, ZFLAGS INTEGER, ZDRAFTMESSAGE INTEGER, ZEXTRA '
          u'INTEGER, ZLASTMESSAGE INTEGER, ZUSER INTEGER, ZDATEUPDATED '
          u'TIMESTAMP )'),
      u'ZKIKCHATEXTRA': (
          u'CREATE TABLE ZKIKCHATEXTRA ( Z_PK INTEGER PRIMARY KEY, Z_ENT '
          u'INTEGER, Z_OPT INTEGER, ZCHAT INTEGER, ZLASTSEENMESSAGE INTEGER, '
          u'ZMUTEDTIMESTAMP TIMESTAMP )'),
      u'ZKIKMESSAGE': (
          u'CREATE TABLE ZKIKMESSAGE ( Z_PK INTEGER PRIMARY KEY, Z_ENT '
          u'INTEGER, Z_OPT INTEGER, ZFLAGS INTEGER, ZINTERNALID INTEGER, '
          u'ZSTATE INTEGER, ZSYSTEMSTATE INTEGER, ZTYPE INTEGER, ZCHATEXTRA '
          u'INTEGER, ZDRAFTMESSAGECHAT INTEGER, ZLASTMESSAGECHAT INTEGER, '
          u'ZLASTMESSAGEUSER INTEGER, ZUSER INTEGER, ZRECEIVEDTIMESTAMP '
          u'TIMESTAMP, ZTIMESTAMP TIMESTAMP, ZBODY VARCHAR, ZSTANZAID VARCHAR, '
          u'ZRENDERINSTRUCTIONSET BLOB )'),
      u'ZKIKUSER': (
          u'CREATE TABLE ZKIKUSER ( Z_PK INTEGER PRIMARY KEY, Z_ENT INTEGER, '
          u'Z_OPT INTEGER, ZADDRESSBOOKID INTEGER, ZFLAGS INTEGER, ZINTERNALID '
          u'INTEGER, ZPRESENCE INTEGER, ZTYPE INTEGER, ZCHATUSER INTEGER, '
          u'ZEXTRA INTEGER, ZLASTMESSAGE INTEGER, ZDISPLAYNAME VARCHAR, '
          u'ZDISPLAYNAMEASCII VARCHAR, ZEMAIL VARCHAR, ZFIRSTNAME VARCHAR, '
          u'ZGROUPTAG VARCHAR, ZJID VARCHAR, ZLASTNAME VARCHAR, ZPPTIMESTAMP '
          u'VARCHAR, ZPPURL VARCHAR, ZSTATUS VARCHAR, ZUSERNAME VARCHAR, '
          u'ZCONTENTLINKSPROTODATA BLOB )'),
      u'ZKIKUSEREXTRA': (
          u'CREATE TABLE ZKIKUSEREXTRA ( Z_PK INTEGER PRIMARY KEY, Z_ENT '
          u'INTEGER, Z_OPT INTEGER, ZLOCALFLAGS INTEGER, ZUSER INTEGER, '
          u'ZPUBLICMESSAGINGKEY BLOB )')}]

  def ParseMessageRow(self, parser_mediator, row, query=None, **unused_kwargs):
    """Parses a message row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row (sqlite3.Row): row.
      query (Optional[str]): query.
    """
    # Note that pysqlite does not accept a Unicode string in row['string'] and
    # will raise "IndexError: Index must be int or string".

    event_data = KikIOSMessageEventData()
    event_data.body = row['ZBODY']
    event_data.displayname = row['ZDISPLAYNAME']
    event_data.message_status = row['ZSTATE']
    event_data.message_type = row['ZTYPE']
    event_data.offset = row['id']
    event_data.query = query
    event_data.username = row['ZUSERNAME']

    # Convert the floating point value to an integer.
    timestamp = int(row['ZRECEIVEDTIMESTAMP'])
    date_time = dfdatetime_cocoa_time.CocoaTime(timestamp=timestamp)
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_CREATION)
    parser_mediator.ProduceEventWithEventData(event, event_data)


sqlite.SQLiteParser.RegisterPlugin(KikIOSPlugin)



# -*- coding: utf-8 -*-
"""This file contains the Windows NT Known Folder identifier definitions."""

# For now ignore the line too long errors.
# pylint: disable=line-too-long

# For now copied from:
# https://code.google.com/p/libfwsi/wiki/KnownFolderIdentifiers

# TODO: store these in a database or equiv.

DESCRIPTIONS = {
    u'008ca0b1-55b4-4c56-b8a8-4de4b299d3be': u'Account Pictures',
    u'00bcfc5a-ed94-4e48-96a1-3f6217f21990': u'Roaming Tiles',
    u'0139d44e-6afe-49f2-8690-3dafcae6ffb8': u'(Common) Programs',
    u'0482af6c-08f1-4c34-8c90-e17ec98b1e17': u'Public Account Pictures',
    u'054fae61-4dd8-4787-80b6-090220c4b700': u'Game Explorer (Game Tasks)',
    u'0762d272-c50a-4bb0-a382-697dcd729b80': u'Users (User Profiles)',
    u'0ac0837c-bbf8-452a-850d-79d08e667ca7': u'Computer (My Computer)',
    u'0d4c3db6-03a3-462f-a0e6-08924c41b5d4': u'History',
    u'0f214138-b1d3-4a90-bba9-27cbc0c5389a': u'Sync Setup',
    u'15ca69b3-30ee-49c1-ace1-6b5ec372afb5': u'Sample Playlists',
    u'1777f761-68ad-4d8a-87bd-30b759fa33dd': u'Favorites',
    u'18989b1d-99b5-455b-841c-ab7c74e4ddfc': u'Videos (My Video)',
    u'190337d1-b8ca-4121-a639-6d472d16972a': u'Search Results (Search Home)',
    u'1a6fdba2-f42d-4358-a798-b74d745926c5': u'Recorded TV',
    u'1ac14e77-02e7-4e5d-b744-2eb1ae5198b7': u'System32 (System)',
    u'1b3ea5dc-b587-4786-b4ef-bd1dc332aeae': u'Libraries',
    u'1e87508d-89c2-42f0-8a7e-645a0f50ca58': u'Applications',
    u'2112ab0a-c86a-4ffe-a368-0de96e47012e': u'Music',
    u'2400183a-6185-49fb-a2d8-4a392a602ba3': u'Public Videos (Common Video)',
    u'24d89e24-2f19-4534-9dde-6a6671fbb8fe': u'One Drive Documents',
    u'289a9a43-be44-4057-a41b-587a76d7e7f9': u'Sync Results',
    u'2a00375e-224c-49de-b8d1-440df7ef3ddc': u'Localized Resources (Directory)',
    u'2b0f765d-c0e9-4171-908e-08a611b84ff6': u'Cookies',
    u'2c36c0aa-5812-4b87-bfd0-4cd0dfb19b39': u'Original Images',
    u'3214fab5-9757-4298-bb61-92a9deaa44ff': u'Public Music (Common Music)',
    u'339719b5-8c47-4894-94c2-d8f77add44a6': u'One Drive Pictures',
    u'33e28130-4e1e-4676-835a-98395c3bc3bb': u'Pictures (My Pictures)',
    u'352481e8-33be-4251-ba85-6007caedcf9d': u'Internet Cache (Temporary Internet Files)',
    u'374de290-123f-4565-9164-39c4925e467b': u'Downloads',
    u'3d644c9b-1fb8-4f30-9b45-f670235f79c0': u'Public Downloads (Common Downloads)',
    u'3eb685db-65f9-4cf6-a03a-e3ef65729f3d': u'Roaming Application Data (Roaming)',
    u'43668bf8-c14e-49b2-97c9-747784d784b7': u'Sync Center (Sync Manager)',
    u'48daf80b-e6cf-4f4e-b800-0e69d84ee384': u'Libraries',
    u'491e922f-5643-4af4-a7eb-4e7a138d8174': u'Videos',
    u'4bd8d571-6d19-48d3-be97-422220080e43': u'Music (My Music)',
    u'4bfefb45-347d-4006-a5be-ac0cb0567192': u'Conflicts',
    u'4c5c32ff-bb9d-43b0-b5b4-2d72e54eaaa4': u'Saved Games',
    u'4d9f7874-4e0c-4904-967b-40b0d20c3e4b': u'Internet (The Internet)',
    u'52528a6b-b9e3-4add-b60d-588c2dba842d': u'Homegroup',
    u'52a4f021-7b75-48a9-9f6b-4b87a210bc8f': u'Quick Launch',
    u'56784854-c6cb-462b-8169-88e350acb882': u'Contacts',
    u'5b3749ad-b49f-49c1-83eb-15370fbd4882': u'Tree Properties',
    u'5cd7aee2-2219-4a67-b85d-6c9ce15660cb': u'Programs',
    u'5ce4a5e9-e4eb-479d-b89f-130c02886155': u'Device Metadata Store',
    u'5e6c858f-0e22-4760-9afe-ea3317b67173': u'Profile (User\'s name)',
    u'625b53c3-ab48-4ec1-ba1f-a1ef4146fc19': u'Start Menu',
    u'62ab5d82-fdc1-4dc3-a9dd-070d1d495d97': u'Program Data',
    u'6365d5a7-0f0d-45e5-87f6-0da56b6a4f7d': u'Common Files (x64)',
    u'69d2cf90-fc33-4fb7-9a0c-ebb0f0fcb43c': u'Slide Shows (Photo Albums)',
    u'6d809377-6af0-444b-8957-a3773f02200e': u'Program Files (x64)',
    u'6f0cd92b-2e97-45d1-88ff-b0d186b8dedd': u'Network Connections',
    u'724ef170-a42d-4fef-9f26-b60e846fba4f': u'Administrative Tools',
    u'767e6811-49cb-4273-87c2-20f355e1085b': u'One Drive Camera Roll',
    u'76fc4e2d-d6ad-4519-a663-37bd56068185': u'Printers',
    u'7b0db17d-9cd2-4a93-9733-46cc89022e7c': u'Documents',
    u'7b396e54-9ec5-4300-be0a-2482ebae1a26': u'Default Gadgets (Sidebar Default Parts)',
    u'7c5a40ef-a0fb-4bfc-874a-c0f2e0b9fa8e': u'Program Files (x86)',
    u'7d1d3a04-debb-4115-95cf-2f29da2920da': u'Saved Searches (Searches)',
    u'7e636bfe-dfa9-4d5e-b456-d7b39851d8a9': u'Templates',
    u'82a5ea35-d9cd-47c5-9629-e15d2f714e6e': u'(Common) Startup',
    u'82a74aeb-aeb4-465c-a014-d097ee346d63': u'Control Panel',
    u'859ead94-2e85-48ad-a71a-0969cb56a6cd': u'Sample Videos',
    u'8983036c-27c0-404b-8f08-102d10dcfd74': u'Send To',
    u'8ad10c31-2adb-4296-a8f7-e4701232c972': u'Resources (Resources Directory)',
    u'905e63b6-c1bf-494e-b29c-65b732d3d21a': u'Program Files',
    u'9274bd8d-cfd1-41c3-b35e-b13f55a758f4': u'Printer Shortcuts (PrintHood)',
    u'98ec0e18-2098-4d44-8644-66979315a281': u'Microsoft Office Outlook (MAPI)',
    u'9b74b6a3-0dfd-4f11-9e78-5f7800f2e772': u'User\'s name',
    u'9e3995ab-1f9c-4f13-b827-48b24b6c7174': u'User Pinned',
    u'9e52ab10-f80d-49df-acb8-4330f5687855': u'Temporary Burn Folder (CD Burning)',
    u'a302545d-deff-464b-abe8-61c8648d939b': u'Libraries',
    u'a305ce99-f527-492b-8b1a-7e76fa98d6e4': u'Installed Updates (Application Updates)',
    u'a3918781-e5f2-4890-b3d9-a7e54332328c': u'Application Shortcuts',
    u'a4115719-d62e-491d-aa7c-e74b8be3b067': u'(Common) Start Menu',
    u'a520a1a4-1780-4ff6-bd18-167343c5af16': u'Local Application Data Low (Local Low)',
    u'a52bba46-e9e1-435f-b3d9-28daa648c0f6': u'One Drive',
    u'a63293e8-664e-48db-a079-df759e0509f7': u'Templates',
    u'a75d362e-50fc-4fb7-ac2c-a8beaa314493': u'Gadgets (Sidebar Parts)',
    u'a77f5d77-2e2b-44c3-a6a2-aba601054a51': u'Programs',
    u'a990ae9f-a03b-4e80-94bc-9912d7504104': u'Pictures',
    u'aaa8d5a5-f1d6-4259-baa8-78e7ef60835e': u'Roamed Tile Images',
    u'ab5fb87b-7ce2-4f83-915d-550846c9537b': u'Camera Roll',
    u'ae50c081-ebd2-438a-8655-8a092e34987a': u'Recent (Recent Items)',
    u'b250c668-f57d-4ee1-a63c-290ee7d1aa1f': u'Sample Music',
    u'b4bfcc3a-db2c-424c-b029-7fe99a87c641': u'Desktop',
    u'b6ebfb86-6907-413c-9af7-4fc2abf07cc5': u'Public Pictures (Common Pictures)',
    u'b7534046-3ecb-4c18-be4e-64cd4cb7d6ac': u'Recycle Bin (Bit Bucket)',
    u'b7bede81-df94-4682-a7d8-57a52620b86f': u'Screenshots',
    u'b94237e7-57ac-4347-9151-b08c6c32d1f7': u'(Common) Templates',
    u'b97d20bb-f46a-4c97-ba10-5e3608430854': u'Startup',
    u'bcb5256f-79f6-4cee-b725-dc34e402fd46': u'Implicit Application Shortcuts',
    u'bcbd3057-ca5c-4622-b42d-bc56db0ae516': u'Programs',
    u'bd85e001-112e-431e-983b-7b15ac09fff1': u'Recorded TV',
    u'bfb9d5e0-c6a9-404c-b2b2-ae6db6af4968': u'Links',
    u'c1bae2d0-10df-4334-bedd-7aa20b227a9d': u'(Common) OEM Links',
    u'c4900540-2379-4c75-844b-64e6faf8716b': u'Sample Pictures',
    u'c4aa340d-f20f-4863-afef-f87ef2e6ba25': u'Public Desktop (Common Desktop)',
    u'c5abbf53-e17f-4121-8900-86626fc2c973': u'Network Shortcuts (NetHood)',
    u'c870044b-f49e-4126-a9c3-b52a1ff411e8': u'Ringtones',
    u'cac52c1a-b53d-4edc-92d7-6b2e8ac19434': u'Games',
    u'd0384e7d-bac3-4797-8f14-cba229b392b5': u'(Common) Administrative Tools',
    u'd20beec4-5ca8-4905-ae3b-bf251ea09b53': u'Network (Places)',
    u'd65231b0-b2f1-4857-a4ce-a8e7c6ea7d27': u'System32 (x86)',
    u'd9dc8a3b-b784-432e-a781-5a1130a75963': u'History',
    u'de61d971-5ebc-4f02-a3a9-6c82895e5c04': u'Add New Programs (Get Programs)',
    u'de92c1c7-837f-4f69-a3bb-86e631204a23': u'Playlists',
    u'de974d24-d9c6-4d3e-bf91-f4455120b917': u'Common Files (x86)',
    u'debf2536-e1a8-4c59-b6a2-414586476aea': u'Game Explorer (Public Game Tasks)',
    u'df7266ac-9274-4867-8d55-3bd661de872d': u'Programs and Features (Change and Remove Programs)',
    u'dfdf76a2-c82a-4d63-906a-5644ac457385': u'Public',
    u'e555ab60-153b-4d17-9f04-a5fe99fc15ec': u'Ringtones',
    u'ed4824af-dce4-45a8-81e2-fc7965083634': u'Public Documents (Common Documents)',
    u'ee32e446-31ca-4aba-814f-a5ebd2fd6d5e': u'Offline Files (CSC)',
    u'f1b32785-6fba-4fcf-9d55-7b8e7f157091': u'Local Application Data',
    u'f38bf404-1d43-42f2-9305-67de0b28fc23': u'Windows',
    u'f3ce0f7c-4901-4acc-8648-d5d44b04ef8f': u'User\'s Files',
    u'f7f1ed05-9f6d-47a2-aaae-29d317c6f066': u'Common Files',
    u'fd228cb7-ae11-4ae3-864c-16f3910ab8fe': u'Fonts',
    u'fdd39ad0-238f-46af-adb4-6c85480369c7': u'Documents (Personal)',
}

PATHS = {
    u'008ca0b1-55b4-4c56-b8a8-4de4b299d3be': u'%APPDATA%\\Microsoft\\Windows\\AccountPictures',
    u'00bcfc5a-ed94-4e48-96a1-3f6217f21990': u'%LOCALAPPDATA%\\Microsoft\\Windows\\RoamingTiles',
    u'0139d44e-6afe-49f2-8690-3dafcae6ffb8': u'%ALLUSERSPROFILE%\\Microsoft\\Windows\\Start Menu\\Programs',
    u'0482af6c-08f1-4c34-8c90-e17ec98b1e17': u'%PUBLIC%\\AccountPictures',
    u'054fae61-4dd8-4787-80b6-090220c4b700': u'%LOCALAPPDATA%\\Microsoft\\Windows\\GameExplorer',
    u'0762d272-c50a-4bb0-a382-697dcd729b80': u'%SYSTEMDRIVE%\\Users',
    u'0ac0837c-bbf8-452a-850d-79d08e667ca7': u'',
    u'0d4c3db6-03a3-462f-a0e6-08924c41b5d4': u'%LOCALAPPDATA%\\Microsoft\\Windows\\ConnectedSearch\\History',
    u'0f214138-b1d3-4a90-bba9-27cbc0c5389a': u'',
    u'15ca69b3-30ee-49c1-ace1-6b5ec372afb5': u'%PUBLIC%\\Music\\Sample Playlists',
    u'1777f761-68ad-4d8a-87bd-30b759fa33dd': u'%USERPROFILE%\\Favorites',
    u'18989b1d-99b5-455b-841c-ab7c74e4ddfc': u'%USERPROFILE%\\Videos',
    u'190337d1-b8ca-4121-a639-6d472d16972a': u'',
    u'1a6fdba2-f42d-4358-a798-b74d745926c5': u'%PUBLIC%\\RecordedTV.library-ms',
    u'1ac14e77-02e7-4e5d-b744-2eb1ae5198b7': u'%WINDIR%\\System32',
    u'1b3ea5dc-b587-4786-b4ef-bd1dc332aeae': u'%APPDATA%\\Microsoft\\Windows\\Libraries',
    u'1e87508d-89c2-42f0-8a7e-645a0f50ca58': u'',
    u'2112ab0a-c86a-4ffe-a368-0de96e47012e': u'%APPDATA%\\Microsoft\\Windows\\Libraries\\Music.library-ms',
    u'2400183a-6185-49fb-a2d8-4a392a602ba3': u'%PUBLIC%\\Videos',
    u'24d89e24-2f19-4534-9dde-6a6671fbb8fe': u'%USERPROFILE%\\OneDrive\\Documents',
    u'289a9a43-be44-4057-a41b-587a76d7e7f9': u'',
    u'2a00375e-224c-49de-b8d1-440df7ef3ddc': u'%WINDIR%\\resources\\%CODEPAGE%',
    u'2b0f765d-c0e9-4171-908e-08a611b84ff6': u'%APPDATA%\\Microsoft\\Windows\\Cookies',
    u'2c36c0aa-5812-4b87-bfd0-4cd0dfb19b39': u'%LOCALAPPDATA%\\Microsoft\\Windows Photo Gallery\\Original Images',
    u'3214fab5-9757-4298-bb61-92a9deaa44ff': u'%PUBLIC%\\Music',
    u'339719b5-8c47-4894-94c2-d8f77add44a6': u'%USERPROFILE%\\OneDrive\\Pictures',
    u'33e28130-4e1e-4676-835a-98395c3bc3bb': u'%USERPROFILE%\\Pictures',
    u'352481e8-33be-4251-ba85-6007caedcf9d': u'%LOCALAPPDATA%\\Microsoft\\Windows\\Temporary Internet Files',
    u'374de290-123f-4565-9164-39c4925e467b': u'%USERPROFILE%\\Downloads',
    u'3d644c9b-1fb8-4f30-9b45-f670235f79c0': u'%PUBLIC%\\Downloads',
    u'3eb685db-65f9-4cf6-a03a-e3ef65729f3d': u'%USERPROFILE%\\AppData\\Roaming',
    u'43668bf8-c14e-49b2-97c9-747784d784b7': u'',
    u'48daf80b-e6cf-4f4e-b800-0e69d84ee384': u'%ALLUSERSPROFILE%\\Microsoft\\Windows\\Libraries',
    u'491e922f-5643-4af4-a7eb-4e7a138d8174': u'%APPDATA%\\Microsoft\\Windows\\Libraries\\Videos.library-ms',
    u'4bd8d571-6d19-48d3-be97-422220080e43': u'%USERPROFILE%\\Music',
    u'4bfefb45-347d-4006-a5be-ac0cb0567192': u'',
    u'4c5c32ff-bb9d-43b0-b5b4-2d72e54eaaa4': u'%USERPROFILE%\\Saved Games',
    u'4d9f7874-4e0c-4904-967b-40b0d20c3e4b': u'',
    u'52528a6b-b9e3-4add-b60d-588c2dba842d': u'',
    u'52a4f021-7b75-48a9-9f6b-4b87a210bc8f': u'%APPDATA%\\Microsoft\\Internet Explorer\\Quick Launch',
    u'56784854-c6cb-462b-8169-88e350acb882': u'',
    u'5b3749ad-b49f-49c1-83eb-15370fbd4882': u'',
    u'5cd7aee2-2219-4a67-b85d-6c9ce15660cb': u'%LOCALAPPDATA%\\Programs',
    u'5ce4a5e9-e4eb-479d-b89f-130c02886155': u'%ALLUSERSPROFILE%\\Microsoft\\Windows\\DeviceMetadataStore',
    u'5e6c858f-0e22-4760-9afe-ea3317b67173': u'%SYSTEMDRIVE%\\Users\\%USERNAME%',
    u'625b53c3-ab48-4ec1-ba1f-a1ef4146fc19': u'%APPDATA%\\Microsoft\\Windows\\Start Menu',
    u'62ab5d82-fdc1-4dc3-a9dd-070d1d495d97': u'%SYSTEMDRIVE%\\ProgramData',
    u'6365d5a7-0f0d-45e5-87f6-0da56b6a4f7d': u'%PROGRAMFILES%\\Common Files',
    u'69d2cf90-fc33-4fb7-9a0c-ebb0f0fcb43c': u'%USERPROFILE%\\Pictures\\Slide Shows',
    u'6d809377-6af0-444b-8957-a3773f02200e': u'%SYSTEMDRIVE%\\Program Files',
    u'6f0cd92b-2e97-45d1-88ff-b0d186b8dedd': u'',
    u'724ef170-a42d-4fef-9f26-b60e846fba4f': u'%APPDATA%\\Microsoft\\Windows\\Start Menu\\Programs\\Administrative Tools',
    u'767e6811-49cb-4273-87c2-20f355e1085b': u'%USERPROFILE%\\OneDrive\\Pictures\\Camera Roll',
    u'76fc4e2d-d6ad-4519-a663-37bd56068185': u'',
    u'7b0db17d-9cd2-4a93-9733-46cc89022e7c': u'%APPDATA%\\Microsoft\\Windows\\Libraries\\Documents.library-ms',
    u'7b396e54-9ec5-4300-be0a-2482ebae1a26': u'%PROGRAMFILES%\\Windows Sidebar\\Gadgets',
    u'7c5a40ef-a0fb-4bfc-874a-c0f2e0b9fa8e': u'%PROGRAMFILES% (%SYSTEMDRIVE%\\Program Files)',
    u'7d1d3a04-debb-4115-95cf-2f29da2920da': u'%USERPROFILE%\\Searches',
    u'7e636bfe-dfa9-4d5e-b456-d7b39851d8a9': u'%LOCALAPPDATA%\\Microsoft\\Windows\\ConnectedSearch\\Templates',
    u'82a5ea35-d9cd-47c5-9629-e15d2f714e6e': u'%ALLUSERSPROFILE%\\Microsoft\\Windows\\Start Menu\\Programs\\StartUp',
    u'82a74aeb-aeb4-465c-a014-d097ee346d63': u'',
    u'859ead94-2e85-48ad-a71a-0969cb56a6cd': u'%PUBLIC%\\Videos\\Sample Videos',
    u'8983036c-27c0-404b-8f08-102d10dcfd74': u'%APPDATA%\\Microsoft\\Windows\\SendTo',
    u'8ad10c31-2adb-4296-a8f7-e4701232c972': u'%WINDIR%\\Resources',
    u'905e63b6-c1bf-494e-b29c-65b732d3d21a': u'%SYSTEMDRIVE%\\Program Files',
    u'9274bd8d-cfd1-41c3-b35e-b13f55a758f4': u'%APPDATA%\\Microsoft\\Windows\\Printer Shortcuts',
    u'98ec0e18-2098-4d44-8644-66979315a281': u'',
    u'9b74b6a3-0dfd-4f11-9e78-5f7800f2e772': u'',
    u'9e3995ab-1f9c-4f13-b827-48b24b6c7174': u'%APPDATA%\\Microsoft\\Internet Explorer\\Quick Launch\\User Pinned',
    u'9e52ab10-f80d-49df-acb8-4330f5687855': u'%LOCALAPPDATA%\\Microsoft\\Windows\\Burn\\Burn',
    u'a302545d-deff-464b-abe8-61c8648d939b': u'',
    u'a305ce99-f527-492b-8b1a-7e76fa98d6e4': u'',
    u'a3918781-e5f2-4890-b3d9-a7e54332328c': u'%LOCALAPPDATA%\\Microsoft\\Windows\\Application Shortcuts',
    u'a4115719-d62e-491d-aa7c-e74b8be3b067': u'%ALLUSERSPROFILE%\\Microsoft\\Windows\\Start Menu',
    u'a520a1a4-1780-4ff6-bd18-167343c5af16': u'%USERPROFILE%\\AppData\\LocalLow',
    u'a52bba46-e9e1-435f-b3d9-28daa648c0f6': u'%USERPROFILE%\\OneDrive',
    u'a63293e8-664e-48db-a079-df759e0509f7': u'%APPDATA%\\Microsoft\\Windows\\Templates',
    u'a75d362e-50fc-4fb7-ac2c-a8beaa314493': u'%LOCALAPPDATA%\\Microsoft\\Windows Sidebar\\Gadgets',
    u'a77f5d77-2e2b-44c3-a6a2-aba601054a51': u'%APPDATA%\\Microsoft\\Windows\\Start Menu\\Programs',
    u'a990ae9f-a03b-4e80-94bc-9912d7504104': u'%APPDATA%\\Microsoft\\Windows\\Libraries\\Pictures.library-ms',
    u'aaa8d5a5-f1d6-4259-baa8-78e7ef60835e': u'%LOCALAPPDATA%\\Microsoft\\Windows\\RoamedTileImages',
    u'ab5fb87b-7ce2-4f83-915d-550846c9537b': u'%USERPROFILE%\\Pictures\\Camera Roll',
    u'ae50c081-ebd2-438a-8655-8a092e34987a': u'%APPDATA%\\Microsoft\\Windows\\Recent',
    u'b250c668-f57d-4ee1-a63c-290ee7d1aa1f': u'%PUBLIC%\\Music\\Sample Music',
    u'b4bfcc3a-db2c-424c-b029-7fe99a87c641': u'%USERPROFILE%\\Desktop',
    u'b6ebfb86-6907-413c-9af7-4fc2abf07cc5': u'%PUBLIC%\\Pictures',
    u'b7534046-3ecb-4c18-be4e-64cd4cb7d6ac': u'',
    u'b7bede81-df94-4682-a7d8-57a52620b86f': u'%USERPROFILE%\\Pictures\\Screenshots',
    u'b94237e7-57ac-4347-9151-b08c6c32d1f7': u'%ALLUSERSPROFILE%\\Microsoft\\Windows\\Templates',
    u'b97d20bb-f46a-4c97-ba10-5e3608430854': u'%APPDATA%\\Microsoft\\Windows\\Start Menu\\Programs\\StartUp',
    u'bcb5256f-79f6-4cee-b725-dc34e402fd46': u'%APPDATA%\\Microsoft\\Internet Explorer\\Quick Launch\\User Pinned\\ImplicitAppShortcuts',
    u'bcbd3057-ca5c-4622-b42d-bc56db0ae516': u'%LOCALAPPDATA%\\Programs\\Common',
    u'bd85e001-112e-431e-983b-7b15ac09fff1': u'',
    u'bfb9d5e0-c6a9-404c-b2b2-ae6db6af4968': u'%USERPROFILE%\\Links',
    u'c1bae2d0-10df-4334-bedd-7aa20b227a9d': u'%ALLUSERSPROFILE%\\OEM Links',
    u'c4900540-2379-4c75-844b-64e6faf8716b': u'%PUBLIC%\\Pictures\\Sample Pictures',
    u'c4aa340d-f20f-4863-afef-f87ef2e6ba25': u'%PUBLIC%\\Desktop',
    u'c5abbf53-e17f-4121-8900-86626fc2c973': u'%APPDATA%\\Microsoft\\Windows\\Network Shortcuts',
    u'c870044b-f49e-4126-a9c3-b52a1ff411e8': u'%LOCALAPPDATA%\\Microsoft\\Windows\\Ringtones',
    u'cac52c1a-b53d-4edc-92d7-6b2e8ac19434': u'',
    u'd0384e7d-bac3-4797-8f14-cba229b392b5': u'%ALLUSERSPROFILE%\\Microsoft\\Windows\\Start Menu\\Programs\\Administrative Tools',
    u'd20beec4-5ca8-4905-ae3b-bf251ea09b53': u'',
    u'd65231b0-b2f1-4857-a4ce-a8e7c6ea7d27': u'%WINDIR%\\system32',
    u'd9dc8a3b-b784-432e-a781-5a1130a75963': u'%LOCALAPPDATA%\\Microsoft\\Windows\\History',
    u'de61d971-5ebc-4f02-a3a9-6c82895e5c04': u'',
    u'de92c1c7-837f-4f69-a3bb-86e631204a23': u'%USERPROFILE%\\Music\\Playlists',
    u'de974d24-d9c6-4d3e-bf91-f4455120b917': u'%PROGRAMFILES%\\Common Files',
    u'debf2536-e1a8-4c59-b6a2-414586476aea': u'%ALLUSERSPROFILE%\\Microsoft\\Windows\\GameExplorer',
    u'df7266ac-9274-4867-8d55-3bd661de872d': u'',
    u'dfdf76a2-c82a-4d63-906a-5644ac457385': u'%SYSTEMDRIVE%\\Users\\Public',
    u'e555ab60-153b-4d17-9f04-a5fe99fc15ec': u'%ALLUSERSPROFILE%\\Microsoft\\Windows\\Ringtones',
    u'ed4824af-dce4-45a8-81e2-fc7965083634': u'%PUBLIC%\\Documents',
    u'ee32e446-31ca-4aba-814f-a5ebd2fd6d5e': u'',
    u'f1b32785-6fba-4fcf-9d55-7b8e7f157091': u'%USERPROFILE%\\AppData\\Local',
    u'f38bf404-1d43-42f2-9305-67de0b28fc23': u'%WINDIR%',
    u'f3ce0f7c-4901-4acc-8648-d5d44b04ef8f': u'',
    u'f7f1ed05-9f6d-47a2-aaae-29d317c6f066': u'%PROGRAMFILES%\\Common Files',
    u'fd228cb7-ae11-4ae3-864c-16f3910ab8fe': u'%WINDIR%\\Fonts',
    u'fdd39ad0-238f-46af-adb4-6c85480369c7': u'%USERPROFILE%\\Documents',
}



# -*- coding: utf-8 -*-
"""The CLI arguments helper manager objects."""

from plaso.lib import errors


class ArgumentHelperManager(object):
  """Class that implements the CLI argument helper manager."""

  _helper_classes = {}

  @classmethod
  def AddCommandLineArguments(
      cls, argument_group, category=None, names=None):
    """Adds command line arguments to a configuration object.

    Args:
      argument_group (argparse._ArgumentGroup|argparse.ArgumentParser):
          argparse group.
      category (Optional[str]): category of helpers to apply to
          the group, such as storage, output, where None will apply the
          arguments to all helpers. The category can be used to add arguments
          to a specific group of registered helpers.
      names (Optional[list[str]]): names of argument helpers to apply,
          where None will apply the arguments to all helpers.
    """
    # Process the helper classes in alphabetical order this is needed to
    # keep the argument order consistent.
    for helper_name, helper_class in sorted(cls._helper_classes.items()):
      if ((category and helper_class.CATEGORY != category) or
          (names and helper_name not in names)):
        continue

      helper_class.AddArguments(argument_group)

  @classmethod
  def DeregisterHelper(cls, helper_class):
    """Deregisters a helper class.

    The helper classes are identified based on their lower case name.

    Args:
      helper_class (type): class object of the argument helper.

    Raises:
      KeyError: if helper class is not set for the corresponding name.
    """
    helper_name = helper_class.NAME.lower()
    if helper_name not in cls._helper_classes:
      raise KeyError(u'Helper class not set for name: {0:s}.'.format(
          helper_class.NAME))

    del cls._helper_classes[helper_name]

  @classmethod
  def ParseOptions(cls, options, config_object, category=None, names=None):
    """Parses and validates arguments using the appropriate helpers.

    Args:
      options (argparse.Namespace): parser options.
      config_object (object): object to be configured by an argument helper.
      category (Optional[str]): category of helpers to apply to
          the group, such as storage, output, where None will apply the
          arguments to all helpers. The category can be used to add arguments
          to a specific group of registered helpers.
      names (Optional[list[str]]): names of argument helpers to apply,
          where None will apply the arguments to all helpers.
    """
    for helper_name, helper_class in cls._helper_classes.items():
      if ((category and helper_class.CATEGORY != category) or
          (names and helper_name not in names)):
        continue

      try:
        helper_class.ParseOptions(options, config_object)
      except errors.BadConfigObject:
        pass

  @classmethod
  def RegisterHelper(cls, helper_class):
    """Registers a helper class.

    The helper classes are identified based on their lower case name.

    Args:
      helper_class (type): class object of the argument helper.

    Raises:
      KeyError: if helper class is already set for the corresponding name.
    """
    helper_name = helper_class.NAME.lower()
    if helper_name in cls._helper_classes:
      raise KeyError(u'Helper class already set for name: {0:s}.'.format(
          helper_class.NAME))

    cls._helper_classes[helper_name] = helper_class

  @classmethod
  def RegisterHelpers(cls, helper_classes):
    """Registers helper classes.

    The helper classes are identified based on their lower case name.

    Args:
      helper_classes (list[type]): class objects of the argument helpers.

    Raises:
      KeyError: if helper class is already set for the corresponding name.
    """
    for helper_class in helper_classes:
      cls.RegisterHelper(helper_class)



# -*- coding: utf-8 -*-
"""The logging filter classes."""

import logging


class LoggingFilter(logging.Filter):
  """Logging filter.

  Some libraries, like binplist, introduce excessive amounts of
  logging that clutters the debug logs of plaso, making them
  almost unusable. This class implements a filter designed to make
  the debug logs more clutter-free.
  """

  def filter(self, record):
    """Filter messages sent to the logging infrastructure.

    Returns:
      bool: True if the record should be included in the logging.
    """
    if record.module == u'binplist' and record.levelno < logging.ERROR:
      return False

    return True



# -*- coding: utf-8 -*-
"""Parser for McAfee Anti-Virus Logs.

McAfee AV uses 4 logs to track when scans were run, when virus databases were
updated, and when files match the virus database."""

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import errors
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.parsers import manager
from plaso.parsers import text_parser


class McafeeAVEventData(events.EventData):
  """McAfee AV Log event data.

  Attributes:
    action (str): action.
    filename (str): filename.
    rule (str): rule.
    status (str): status.
    trigger_location (str): trigger loction.
    username (str): username.
  """

  DATA_TYPE = u'av:mcafee:accessprotectionlog'

  def __init__(self):
    """Initializes event data."""
    super(McafeeAVEventData, self).__init__(data_type=self.DATA_TYPE)
    self.action = None
    self.filename = None
    self.rule = None
    self.status = None
    self.trigger_location = None
    self.username = None


class McafeeAccessProtectionParser(text_parser.TextCSVParser):
  """Parses the McAfee AV Access Protection Log."""

  NAME = u'mcafee_protection'
  DESCRIPTION = u'Parser for McAfee AV Access Protection log files.'

  VALUE_SEPARATOR = b'\t'
  COLUMNS = [
      u'date', u'time', u'status', u'username', u'filename',
      u'trigger_location', u'rule', u'action']

  def _ConvertToTimestamp(self, date, time, timezone):
    """Converts date and time values into a timestamp.

    The date and time are made up of two strings, the date and the time,
    separated by a tab. The time is in local time. The month and day can
    be either 1 or 2 characters long, e.g.: 7/30/2013\\t10:22:48 AM

    Args:
      date: a string representing the date.
      time: a string representing the time.
      timezone: a timezone (instance of pytz.timezone) that the date and
                time values represent.

    Returns:
      The timestamp which is an integer containing the number of micro seconds
      since January 1, 1970, 00:00:00 UTC.

    Raises:
      TimestampError: if the timestamp is badly formed or unable to transfer
                      the supplied date and time into a timestamp.
    """
    # TODO: check if this is correct, likely not date or not time
    # is more accurate.
    if not date and not time:
      raise errors.TimestampError(
          u'Unable to extract timestamp from McAfee AV logline.')

    # TODO: Figure out how McAfee sets Day First and use that here.
    # The in-file time format is '07/30/2013\t10:22:48 AM'.
    try:
      time_string = u'{0:s} {1:s}'.format(date, time)
    except UnicodeDecodeError:
      raise errors.TimestampError(u'Unable to form a timestamp string.')

    return timelib.Timestamp.FromTimeString(time_string, timezone=timezone)

  def ParseRow(self, parser_mediator, row_offset, row):
    """Parses a row and extract event objects.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row_offset (int): line number of the row.
      row (dict[str, str]): row of the fields specified in COLUMNS.
    """
    try:
      timestamp = self._ConvertToTimestamp(
          row[u'date'], row[u'time'], parser_mediator.timezone)
    except errors.TimestampError as exception:
      parser_mediator.ProduceExtractionError(
          u'Unable to parse time string: [{0:s} {1:s}] with error {2:s}'.format(
              repr(row[u'date']), repr(row[u'time']), exception))
      return

    if timestamp is None:
      return

    event_data = McafeeAVEventData()
    event_data.action = row[u'action']
    event_data.filename = row[u'filename']
    event_data.offset = row_offset
    event_data.rule = row[u'rule']
    event_data.status = row[u'status']
    event_data.trigger_location = row[u'trigger_location']
    event_data.username = row[u'username']

    event = time_events.TimestampEvent(
        timestamp, definitions.TIME_DESCRIPTION_WRITTEN)
    parser_mediator.ProduceEventWithEventData(event, event_data)

  def VerifyRow(self, parser_mediator, row):
    """Verify that this is a McAfee AV Access Protection Log file.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row (dict[str, str]): row of the fields specified in COLUMNS.

    Returns:
      bool: True if the row is in the expected format, False if not.
    """
    if len(row) != 8:
      return False

    # This file can have a UTF-8 byte-order-marker at the beginning of
    # the first row.
    # TODO: Find out all the code pages this can have.  Asked McAfee 10/31.
    if row[u'date'][0:3] == b'\xef\xbb\xbf':
      row[u'date'] = row[u'date'][3:]
      self.encoding = u'utf-8'

    # Check the date format!
    # If it doesn't parse, then this isn't a McAfee AV Access Protection Log
    try:
      timestamp = self._ConvertToTimestamp(
          row[u'date'], row[u'time'], parser_mediator.timezone)
    except errors.TimestampError:
      return False

    if timestamp is None:
      return False

    # Use the presence of these strings as a backup or in case of partial file.
    if (not u'Access Protection' in row[u'status'] and
        not u'Would be blocked' in row[u'status']):
      return False

    return True


manager.ParsersManager.RegisterParser(McafeeAccessProtectionParser)



# -*- coding: utf-8 -*-
"""The processing engine."""

import logging

from dfvfs.helpers import file_system_searcher
from dfvfs.lib import errors as dfvfs_errors
from dfvfs.path import factory as path_spec_factory
from dfvfs.resolver import resolver as path_spec_resolver

from plaso.containers import sessions
from plaso.engine import knowledge_base
from plaso.engine import processing_status
from plaso.engine import profiler
from plaso.lib import definitions
from plaso.preprocessors import manager as preprocess_manager


class BaseEngine(object):
  """Processing engine interface.

  Attributes:
    knowledge_base (KnowledgeBase): knowledge base.
  """

  # The interval of status updates in number of seconds.
  _STATUS_UPDATE_INTERVAL = 0.5

  def __init__(self):
    """Initializes an engine."""
    super(BaseEngine, self).__init__()
    self._abort = False
    self._processing_status = processing_status.ProcessingStatus()

    self.knowledge_base = knowledge_base.KnowledgeBase()

  def _GuessOS(self, searcher):
    """Tries to determine the underlying operating system.

    Args:
      searcher (dfvfs.FileSystemSearcher): file system searcher.

    Returns:
      str: operating system for example "Windows". This should be one of
          the values in definitions.OPERATING_SYSTEMS.
    """
    find_specs = [
        file_system_searcher.FindSpec(
            location=u'/etc', case_sensitive=False),
        file_system_searcher.FindSpec(
            location=u'/System/Library', case_sensitive=False),
        file_system_searcher.FindSpec(
            location=u'/Windows/System32', case_sensitive=False),
        file_system_searcher.FindSpec(
            location=u'/WINNT/System32', case_sensitive=False),
        file_system_searcher.FindSpec(
            location=u'/WINNT35/System32', case_sensitive=False),
        file_system_searcher.FindSpec(
            location=u'/WTSRV/System32', case_sensitive=False)]

    locations = []
    for path_spec in searcher.Find(find_specs=find_specs):
      relative_path = searcher.GetRelativePath(path_spec)
      if relative_path:
        locations.append(relative_path.lower())

    # We need to check for both forward and backward slashes since the path
    # spec will be OS dependent, as in running the tool on Windows will return
    # Windows paths (backward slash) vs. forward slash on *NIX systems.
    windows_locations = set([
        u'/windows/system32', u'\\windows\\system32', u'/winnt/system32',
        u'\\winnt\\system32', u'/winnt35/system32', u'\\winnt35\\system32',
        u'\\wtsrv\\system32', u'/wtsrv/system32'])

    if windows_locations.intersection(set(locations)):
      return definitions.OPERATING_SYSTEM_WINDOWS

    if u'/system/library' in locations:
      return definitions.OPERATING_SYSTEM_MACOSX

    if u'/etc' in locations:
      return definitions.OPERATING_SYSTEM_LINUX

    return definitions.OPERATING_SYSTEM_UNKNOWN

  @classmethod
  def CreateSession(
      cls, command_line_arguments=None, debug_mode=False,
      filter_file=None, preferred_encoding=u'utf-8',
      preferred_time_zone=None, preferred_year=None):
    """Creates a session attribute containiner.

    Args:
      command_line_arguments (Optional[str]): the command line arguments.
      debug_mode (bool): True if debug mode was enabled.
      filter_file (Optional[str]): path to a file with find specifications.
      preferred_encoding (Optional[str]): preferred encoding.
      preferred_time_zone (Optional[str]): preferred time zone.
      preferred_year (Optional[int]): preferred year.

    Returns:
      Session: session attribute container.
    """
    session = sessions.Session()

    session.command_line_arguments = command_line_arguments
    session.debug_mode = debug_mode
    session.filter_file = filter_file
    session.preferred_encoding = preferred_encoding
    session.preferred_time_zone = preferred_time_zone
    session.preferred_year = preferred_year

    return session

  def GetSourceFileSystem(self, source_path_spec, resolver_context=None):
    """Retrieves the file system of the source.

    Args:
      source_path_spec (dfvfs.PathSpec): path specifications of the sources
          to process.
      resolver_context (dfvfs.Context): resolver context.

    Returns:
      tuple: containing:

        dfvfs.FileSystem: file system
        path.PathSpec: mount point path specification. The mount point path
            specification refers to either a directory or a volume on a storage
            media device or image. It is needed by the dfVFS file system
            searcher (FileSystemSearcher) to indicate the base location of
            the file system.

    Raises:
      RuntimeError: if source file system path specification is not set.
    """
    if not source_path_spec:
      raise RuntimeError(u'Missing source path specification.')

    file_system = path_spec_resolver.Resolver.OpenFileSystem(
        source_path_spec, resolver_context=resolver_context)

    type_indicator = source_path_spec.type_indicator
    if path_spec_factory.Factory.IsSystemLevelTypeIndicator(type_indicator):
      mount_point = source_path_spec
    else:
      mount_point = source_path_spec.parent

    return file_system, mount_point

  def PreprocessSources(
      self, artifacts_registry, source_path_specs, resolver_context=None):
    """Preprocesses the sources.

    Args:
      artifacts_registry (artifacts.ArtifactDefinitionsRegistry]): artifact
          definitions registry.
      source_path_specs (list[dfvfs.PathSpec]): path specifications of
          the sources to process.
      resolver_context (Optional[dfvfs.Context]): resolver context.
    """
    platforms = []
    for source_path_spec in source_path_specs:
      try:
        file_system, mount_point = self.GetSourceFileSystem(
            source_path_spec, resolver_context=resolver_context)
      except (RuntimeError, dfvfs_errors.BackEndError) as exception:
        logging.error(exception)
        continue

      try:
        searcher = file_system_searcher.FileSystemSearcher(
            file_system, mount_point)

        platform = self._GuessOS(searcher)
        if platform != definitions.OPERATING_SYSTEM_UNKNOWN:
          preprocess_manager.PreprocessPluginsManager.RunPlugins(
              artifacts_registry, file_system, mount_point, self.knowledge_base)

          platforms.append(platform)

      finally:
        file_system.Close()

    if platforms:
      logging.info(u'Preprocessing detected platforms: {0:s}'.format(
          u', '.join(platforms)))
      self.knowledge_base.platform = platforms[0]

  @classmethod
  def SupportsGuppyMemoryProfiling(cls):
    """Determines if memory profiling with guppy is supported.

    Returns:
      bool: True if memory profiling with guppy is supported.
    """
    return profiler.GuppyMemoryProfiler.IsSupported()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Zeitgeist activity database plugin."""

import unittest

from plaso.formatters import zeitgeist  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers.sqlite_plugins import zeitgeist

from tests import test_lib as shared_test_lib
from tests.parsers.sqlite_plugins import test_lib


class ZeitgeistActivityDatabasePluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the Zeitgeist activity database plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'activity.sqlite'])
  def testProcess(self):
    """Tests the Process function."""
    plugin = zeitgeist.ZeitgeistActivityDatabasePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        [u'activity.sqlite'], plugin)

    # The sqlite database contains 44 events.
    self.assertEqual(storage_writer.number_of_events, 44)

    events = list(storage_writer.GetEvents())

    # Check the first event.
    event = events[0]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-10-22 08:53:19.477')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_subject_uri = u'application://rhythmbox.desktop'
    self.assertEqual(event.subject_uri, expected_subject_uri)

    expected_message = u'application://rhythmbox.desktop'
    self._TestGetMessageStrings(event, expected_message, expected_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Parser for OLE Compound Files (OLECF)."""

import pyolecf

from plaso.lib import specification
from plaso.parsers import interface
from plaso.parsers import manager


class OLECFParser(interface.FileObjectParser):
  """Parses OLE Compound Files (OLECF)."""

  _INITIAL_FILE_OFFSET = None

  NAME = u'olecf'
  DESCRIPTION = u'Parser for OLE Compound Files (OLECF).'

  _plugin_classes = {}

  @classmethod
  def GetFormatSpecification(cls):
    """Retrieves the format specification.

    Returns:
      FormatSpecification: format specification.
    """
    format_specification = specification.FormatSpecification(cls.NAME)

    # OLECF
    format_specification.AddNewSignature(
        b'\xd0\xcf\x11\xe0\xa1\xb1\x1a\xe1', offset=0)

    # OLECF beta
    format_specification.AddNewSignature(
        b'\x0e\x11\xfc\x0d\xd0\xcf\x11\x0e', offset=0)

    return format_specification

  def ParseFileObject(self, parser_mediator, file_object, **kwargs):
    """Parses an OLE Compound File (OLECF) file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): file-like object.
    """
    olecf_file = pyolecf.file()
    olecf_file.set_ascii_codepage(parser_mediator.codepage)

    try:
      olecf_file.open_file_object(file_object)
    except IOError as exception:
      parser_mediator.ProduceExtractionError(
          u'unable to open file with error: {0:s}'.format(exception))
      return

    root_item = olecf_file.root_item
    if not root_item:
      return

    # Get a list of all items in the root item from the OLECF file.
    item_names = [item.name for item in root_item.sub_items]

    # Compare the list of available plugin objects.
    # We will try to use every plugin against the file (except
    # the default plugin) and run it. Only if none of the plugins
    # works will we use the default plugin.

    item_names = frozenset(item_names)

    try:
      for plugin in self._plugins:
        if item_names < plugin.REQUIRED_ITEMS:
          continue

        try:
          plugin.UpdateChainAndProcess(parser_mediator, root_item=root_item)

        except Exception as exception:  # pylint: disable=broad-except
          parser_mediator.ProduceExtractionError((
              u'plugin: {0:s} unable to parse OLECF file with error: '
              u'{1:s}').format(plugin.NAME, exception))

      if self._default_plugin:
        try:
          self._default_plugin.UpdateChainAndProcess(
              parser_mediator, root_item=root_item)

        except Exception as exception:  # pylint: disable=broad-except
          parser_mediator.ProduceExtractionError((
              u'plugin: {0:s} unable to parse OLECF file with error: '
              u'{1:s}').format(self._default_plugin.NAME, exception))

    finally:
      olecf_file.close()


manager.ParsersManager.RegisterParser(OLECFParser)



# -*- coding: utf-8 -*-
"""The plist event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class PlistFormatter(interface.ConditionalEventFormatter):
  """Formatter for a plist key event."""

  DATA_TYPE = u'plist:key'

  FORMAT_STRING_SEPARATOR = u''

  FORMAT_STRING_PIECES = [
      u'{root}/',
      u'{key}',
      u' {desc}']

  SOURCE_LONG = u'Plist Entry'
  SOURCE_SHORT = u'PLIST'


manager.FormattersManager.RegisterFormatter(PlistFormatter)



# -*- coding: utf-8 -*-
"""This file contains BagMRU Windows Registry plugins (shellbags)."""

import construct

from plaso.containers import time_events
from plaso.containers import windows_events
from plaso.lib import definitions
from plaso.parsers.shared import shell_items
from plaso.parsers import winreg
from plaso.parsers.winreg_plugins import interface


class BagMRUPlugin(interface.WindowsRegistryPlugin):
  """Class that defines a BagMRU Windows Registry plugin."""

  NAME = u'bagmru'
  DESCRIPTION = u'Parser for BagMRU Registry data.'

  FILTERS = frozenset([
      interface.WindowsRegistryKeyPathFilter(
          u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\Shell\\BagMRU'),
      interface.WindowsRegistryKeyPathFilter(
          u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\ShellNoRoam\\'
          u'BagMRU'),
      interface.WindowsRegistryKeyPathFilter(
          u'HKEY_CURRENT_USER\\Software\\Classes\\Local Settings\\Software\\'
          u'Microsoft\\Windows\\Shell\\BagMRU'),
      interface.WindowsRegistryKeyPathFilter(
          u'HKEY_CURRENT_USER\\Software\\Classes\\Local Settings\\Software\\'
          u'Microsoft\\Windows\\ShellNoRoam\\BagMRU')])

  URLS = [
      (u'https://github.com/libyal/winreg-kb/blob/master/documentation/'
       u'MRU%20keys.asciidoc#bagmru-key')]

  _MRULISTEX_ENTRY = construct.ULInt32(u'entry_number')

  _SOURCE_APPEND = u': BagMRU'

  def _ParseMRUListExEntryValue(
      self, parser_mediator, registry_key, entry_index, entry_number,
      values_dict, value_strings, parent_path_segments, codepage=u'cp1252'):
    """Parses the MRUListEx entry value.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key that contains
           the MRUListEx value.
      entry_index (int): MRUListEx entry index.
      entry_number (int): entry number.
      values_dict (dict[str, object]): values of the key.
      value_strings (dict[str, str]): value names and strings.
      parent_path_segments (list[str]): parent shell item path segments.
      codepage (Optional[str]): extended ASCII string codepage.

    Returns:
      str: path segment of the shell item.
    """
    value = registry_key.GetValueByName(u'{0:d}'.format(entry_number))
    path_segment = u'N/A'
    value_string = u''
    if value is None:
      parser_mediator.ProduceExtractionError(
          u'Missing MRUListEx entry value: {0:d} in key: {1:s}.'.format(
              entry_number, registry_key.path))

    elif not value.DataIsBinaryData():
      parser_mediator.ProduceExtractionError(
          u'Non-binary MRUListEx entry value: {0:d} in key: {1:s}.'.format(
              entry_number, registry_key.path))

    elif value.data:
      shell_items_parser = shell_items.ShellItemsParser(registry_key.path)
      shell_items_parser.ParseByteStream(
          parser_mediator, value.data,
          parent_path_segments=parent_path_segments, codepage=codepage)

      path_segment = shell_items_parser.GetUpperPathSegment()
      value_string = shell_items_parser.CopyToPath()

      value_strings[entry_number] = value_string

      value_string = u'Shell item path: {0:s}'.format(value_string)

    value_text = u'Index: {0:d} [MRU Value {1:d}]'.format(
        entry_index + 1, entry_number)

    values_dict[value_text] = value_string

    return path_segment

  def _ParseMRUListExValue(self, parser_mediator, registry_key):
    """Parses the MRUListEx value in a given Registry key.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key that contains
           the MRUListEx value.

    Yields:
      tuple: contains:

        int: MRUListEx index, where 0 is the first index value.
        int: entry number.
    """
    mru_list_value = registry_key.GetValueByName(u'MRUListEx')
    if mru_list_value:
      mrulistex_data = mru_list_value.data
      data_size = len(mrulistex_data)
      _, remainder = divmod(data_size, 4)

      if remainder != 0:
        parser_mediator.ProduceExtractionError((
            u'MRUListEx value data size is not a multitude of 4 '
            u'in MRU key: {0:s}').format(registry_key.path))
        data_size -= remainder

      entry_index = 0
      data_offset = 0
      while data_offset < data_size:
        try:
          entry_number = self._MRULISTEX_ENTRY.parse(
              mrulistex_data[data_offset:])
          yield entry_index, entry_number
        except construct.FieldError:
          parser_mediator.ProduceExtractionError((
              u'Unable to parse MRUListEx value data at offset: {0:d} '
              u'in MRU key: {1:s}').format(data_offset, registry_key.path))

        entry_index += 1
        data_offset += 4

  def _ParseSubKey(
      self, parser_mediator, registry_key, parent_path_segments,
      codepage=u'cp1252'):
    """Extract event objects from a MRUListEx Registry key.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
      parent_path_segments (list[str]): parent shell item path segments.
      codepage (Optional[str]): extended ASCII string codepage.
    """
    entry_numbers = {}
    values_dict = {}
    value_strings = {}

    found_terminator = False
    for index, entry_number in self._ParseMRUListExValue(
        parser_mediator, registry_key):
      if entry_number == 0xffffffff:
        found_terminator = True
        continue

      if found_terminator:
        parser_mediator.ProduceExtractionError((
            u'Found additional MRUListEx entries after terminator '
            u'in key: {0:s}.').format(registry_key.path))

        # Only create one parser error per terminator.
        found_terminator = False

      path_segment = self._ParseMRUListExEntryValue(
          parser_mediator, registry_key, index, entry_number, values_dict,
          value_strings, parent_path_segments, codepage=codepage)

      entry_numbers[entry_number] = path_segment

    event_data = windows_events.WindowsRegistryEventData()
    event_data.key_path = registry_key.path
    event_data.offset = registry_key.offset
    event_data.regvalue = values_dict
    event_data.source_append = self._SOURCE_APPEND
    event_data.urls = self.URLS

    event = time_events.DateTimeValuesEvent(
        registry_key.last_written_time, definitions.TIME_DESCRIPTION_WRITTEN)
    parser_mediator.ProduceEventWithEventData(event, event_data)

    for entry_number, path_segment in iter(entry_numbers.items()):
      sub_key_name = u'{0:d}'.format(entry_number)
      sub_key = registry_key.GetSubkeyByName(sub_key_name)
      if not sub_key:
        parser_mediator.ProduceExtractionError(
            u'Missing BagMRU sub key: {0:d} in key: {1:s}.'.format(
                entry_number, registry_key.path))
        continue

      parent_path_segments.append(path_segment)
      self._ParseSubKey(
          parser_mediator, sub_key, parent_path_segments, codepage=codepage)
      parent_path_segments.pop()

  def ExtractEvents(
      self, parser_mediator, registry_key, codepage=u'cp1252', **kwargs):
    """Extracts events from a Windows Registry key.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
      codepage (Optional[str]): extended ASCII string codepage.
    """
    self._ParseSubKey(parser_mediator, registry_key, [], codepage=codepage)


winreg.WinRegistryParser.RegisterPlugin(BagMRUPlugin)



# -*- coding: utf-8 -*-
"""The hasher interface."""

import abc


class BaseHasher(object):
  """Base class for objects that calculate hashes."""

  NAME = u'base_hasher'
  DESCRIPTION = u'Calculates a digest hash over input data.'

  @abc.abstractmethod
  def GetBinaryDigest(self):
    """Retrieves the digest of the hash function as a binary string.

    Returns:
      bytes: binary hash digest calculated over the data blocks passed to
          Update().
    """

  @abc.abstractmethod
  def GetStringDigest(self):
    """Retrieves the digest of the hash function expressed as a Unicode string.

    Returns:
      str: string hash digest calculated over the data blocks passed to
          Update(). The string consists of printable Unicode characters.
    """

  @abc.abstractmethod
  def Update(self, data):
    """Updates the current state of the hasher with a new block of data.

    Repeated calls to update are equivalent to one single call with the
    concatenation of the arguments.

    Args:
      data(bytes): data with which to update the context of the hasher.
    """



# -*- coding: utf-8 -*-
"""This file contains XChat scrollback log file parser in plaso.

Information updated 06 September 2013.

Besides the logging capability, the XChat IRC client has the option to
record the text for opened tabs. So, when rejoining a particular channel
and/or a particular conversation, XChat will display the last messages
exchanged. This artifact could be present, if not disabled, even if
normal logging is disabled.

From the XChat FAQ (http://xchatdata.net/Using/FAQ):

Q: 'How do I keep text from previous sessions from being displayed when I
join a channel?'
R: 'Starting in XChat 2.8.4, XChat implemented the Scrollback feature which
displays text from the last time you had a particular tab open.
To disable this setting for all channels, Go to Settings -> Preferences
-> Logging and uncheck Display scrollback from previous session.
In XChat 2.8.6, XChat implemented both Per Channel Logging, and
Per Channel Scrollbacks. If you are on 2.8.6 or newer, you can disable
loading scrollback for just one particular tab name by right clicking on
the tab name, selecting Settings, and then unchecking Reload scrollback'

The log file format differs from logging format, but it's quite simple
'T 1232315916 Python interface unloaded'
<T><space><decimal timestamp><space><text><\n>

The time reported in the log is the number of seconds since January 1, 1970
00:00:00 UTC (from source code, time(0)). The <text> part could contain some
'decorators' (bold, underline, colors indication, etc.), so the parser
should strip those control fields.

References
http://xchat.org
"""

import logging

import pyparsing

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.parsers import manager
from plaso.parsers import text_parser


__author__ = 'Francesco Picasso (francesco.picasso@gmail.com)'


class XChatScrollbackEventData(events.EventData):
  """XChat Scrollback line event data.

  Attributes:
    nickname (str): nickname.
    text (str): text sent by nickname service messages.
  """

  DATA_TYPE = u'xchat:scrollback:line'

  def __init__(self):
    """Initializes event data."""
    super(XChatScrollbackEventData, self).__init__(data_type=self.DATA_TYPE)
    self.nickname = None
    self.text = None


class XChatScrollbackParser(text_parser.PyparsingSingleLineTextParser):
  """Parses XChat scrollback log files."""

  NAME = u'xchatscrollback'
  DESCRIPTION = u'Parser for XChat scrollback log files.'

  _ENCODING = u'UTF-8'

  # Define how a log line should look like.
  LOG_LINE = (
      pyparsing.Literal(u'T').suppress() +
      pyparsing.Word(pyparsing.nums).setResultsName(u'timestamp') +
      pyparsing.SkipTo(pyparsing.LineEnd()).setResultsName(u'text'))
  LOG_LINE.parseWithTabs()

  # Define the available log line structures.
  LINE_STRUCTURES = [
      (u'logline', LOG_LINE),
  ]

  # Define for the stripping phase.
  STRIPPER = (
      pyparsing.Word(u'\x03', pyparsing.nums, max=3).suppress() |
      pyparsing.Word(u'\x02\x07\x08\x0f\x16\x1d\x1f', exact=1).suppress())

  # Define the structure for parsing <text> and get <nickname> and <text>
  MSG_NICK_START = pyparsing.Literal(u'<')
  MSG_NICK_END = pyparsing.Literal(u'>')
  MSG_NICK = pyparsing.SkipTo(MSG_NICK_END).setResultsName(u'nickname')
  MSG_ENTRY_NICK = pyparsing.Optional(MSG_NICK_START + MSG_NICK + MSG_NICK_END)
  MSG_ENTRY_TEXT = pyparsing.SkipTo(pyparsing.LineEnd()).setResultsName(u'text')
  MSG_ENTRY = MSG_ENTRY_NICK + MSG_ENTRY_TEXT
  MSG_ENTRY.parseWithTabs()

  def __init__(self):
    """Initializes a parser."""
    super(XChatScrollbackParser, self).__init__()
    self._offset = 0

  def _StripThenGetNicknameAndText(self, text):
    """Strips decorators from text and gets <nickname> if available.

    This method implements the XChat strip_color2 and fe_print_text
    functions, slightly modified to get pure text. From the parsing point
    of view, after having stripped, the code takes everything as is,
    simply replacing tabs with spaces (as the original XChat code).
    So the VerifyStructure plays an important role in checking if
    the source file has the right format, since the method will not raise
    any parse exception and every content will be good.

    Args:
      text (str): text obtained from the log record.

    Returns:
      tuple: contains:

        nickname (str): nickname.
        text (str): text sent by nickname or service messages.
    """
    stripped = self.STRIPPER.transformString(text)
    structure = self.MSG_ENTRY.parseString(stripped)
    text = structure.text.replace(u'\t', u' ')
    return structure.nickname, text

  def ParseRecord(self, parser_mediator, key, structure):
    """Parses a log record structure.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): structure parsed from the log file.
    """
    if key != u'logline':
      logging.warning(
          u'Unable to parse record, unknown structure: {0:s}'.format(key))
      return

    try:
      timestamp = int(structure.timestamp)
    except ValueError:
      logging.debug(u'Invalid timestamp string {0:s}, skipping record'.format(
          structure.timestamp))
      return

    try:
      nickname, text = self._StripThenGetNicknameAndText(structure.text)
    except pyparsing.ParseException:
      logging.debug(u'Error parsing entry at offset {0:d}'.format(self._offset))
      return

    event_data = XChatScrollbackEventData()
    event_data.nickname = nickname
    event_data.offset = self._offset
    event_data.text = text

    date_time = dfdatetime_posix_time.PosixTime(timestamp=timestamp)
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_ADDED)
    parser_mediator.ProduceEventWithEventData(event, event_data)

  def VerifyStructure(self, parser_mediator, line):
    """Verify that this file is a XChat scrollback log file.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      line (str): single line from the text file.

    Returns:
      bool: True if the line was successfully parsed.
    """
    structure = self.LOG_LINE

    try:
      parsed_structure = structure.parseString(line)
    except pyparsing.ParseException:
      logging.debug(u'Not a XChat scrollback log file')
      return False

    try:
      timestamp = int(parsed_structure.timestamp)
    except ValueError:
      logging.debug(
          u'Not a XChat scrollback log file, invalid timestamp string')
      return False

    if not timelib.Timestamp.FromPosixTime(timestamp):
      logging.debug(u'Not a XChat scrollback log file, invalid timestamp')
      return False

    return True


manager.ParsersManager.RegisterParser(XChatScrollbackParser)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Timesketch output class."""

import unittest

from mock import Mock
from mock import MagicMock

from plaso.output import timesketch_out

from tests.output import test_lib


# Mock the imports if timesketch is not available. If timesketch is
# not available the timesketch attribute is set to None in the
# output module.
if timesketch_out.timesketch is None:
  timesketch_mock = Mock()
  timesketch_mock.create_app = MagicMock()

  # Mock out all imports.
  timesketch_out.timesketch = timesketch_mock
  timesketch_out.elastic_exceptions = Mock()
  timesketch_out.current_app = MagicMock()
  timesketch_out.ElasticSearchDataStore = Mock()
  timesketch_out.db_sessions = Mock()
  timesketch_out.SearchIndex = Mock()
  timesketch_out.User = Mock()


class TimesketchTestConfig(object):
  """Config object for the tests."""
  timeline_name = u'Test'
  output_format = u'timesketch'
  index = u''
  show_stats = False
  flush_interval = 1000


class TimesketchOutputModuleTest(test_lib.OutputModuleTestCase):
  """Tests for the Timesketch output class."""

  def setUp(self):
    """Makes preparations before running an individual test."""
    output_mediator = self._CreateOutputMediator()
    self._timesketch_output = timesketch_out.TimesketchOutputModule(
        output_mediator)


  def testMissingParameters(self):
    """Tests the GetMissingArguments function."""
    self.assertListEqual(
        self._timesketch_output.GetMissingArguments(), [u'timeline_name'])

    config = TimesketchTestConfig()

    self._timesketch_output.SetIndexName(config.index)
    self._timesketch_output.SetFlushInterval(config.flush_interval)
    self.assertListEqual(
        self._timesketch_output.GetMissingArguments(), [u'timeline_name'])

    self._timesketch_output.SetTimelineName(config.timeline_name)
    self.assertListEqual(self._timesketch_output.GetMissingArguments(), [])


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""A scan tree-based path filter implementation.

The scan tree is a tree based on multiple paths that contains the path
segments per node. The most significant path segment is at the root and
therefore compared first. More information can be found here:
https://github.com/libyal/libsigscan/wiki/Internals
#scanning-tree-based-signature-scanning

The scan tree is used in the filter to filter provided paths.
"""

from plaso.lib import py2to3


class _PathFilterTable(object):
  """Class that implements a path filter table.

  The path filter table is used to construct a scan tree.
  """

  def __init__(self, paths, ignore_list, path_segment_separator=u'/'):
    """Initializes and builds the path filter table from a list of paths.

    Args:
      paths: a list of strings containing the paths.
      ignore_list: a list of path segment indexes to ignore, where 0 is the
                   index of the first path segment relative from the root.
      path_segment_separator: optional string containing the path segment
                              separator.
    """
    super(_PathFilterTable, self).__init__()
    self._path_segment_separator = path_segment_separator
    self.path_segments_per_index = {}
    self.paths = list(paths)

    for path in self.paths:
      self._AddPathSegments(path, ignore_list)

  def _AddPathSegments(self, path, ignore_list):
    """Adds the path segments to the table.

    Args:
      path: a string containing the path.
      ignore_list: a list of path segment indexes to ignore, where 0 is the
                   index of the first path segment relative from the root.
    """
    path_segments = path.split(self._path_segment_separator)
    for path_segment_index, path_segment in enumerate(path_segments):
      if path_segment_index not in self.path_segments_per_index:
        self.path_segments_per_index[path_segment_index] = {}

      if path_segment_index not in ignore_list:
        path_segments = self.path_segments_per_index[path_segment_index]

        if path_segment not in path_segments:
          path_segments[path_segment] = []

        paths_per_segment_list = path_segments[path_segment]
        paths_per_segment_list.append(path)

  def GetPathIndexesList(self, path_segment_index, path_segment):
    """Retrieves the path indexes list.

    Args:
      path_segment_index: an integer containing the path segment index.
      path_segment: a string containing the path segment.

    Returns:
      A list of integers containing the path indexes.
    """
    return self.path_segments_per_index[path_segment_index][path_segment]

  def GetPathSegments(self, path_segment_index):
    """Retrieves the path segments for a specific path segment index.

    Args:
      path_segment_index: an integer containing the path segment index.

    Returns:
      A dictionary containing the paths list per path segment.
    """
    return self.path_segments_per_index[path_segment_index]

  def ToDebugString(self):
    """Converts the path filter table into a debug string."""
    text_parts = [u'Path segment index\tPath segments(s)']
    for index, path_segments in self.path_segments_per_index.items():
      text_parts.append(u'{0:d}\t\t\t[{1:s}]'.format(
          index, u', '.join(path_segments)))

    text_parts.append(u'')

    return u'\n'.join(text_parts)


class _PathSegmentWeights(object):
  """Class that implements path segment weights.

  The path segment weights are used to determine the most significant
  path segment per index. The path segment weights are used to construct
  a scan tree.
  """

  def __init__(self):
    """Initializes the path segment weights."""
    super(_PathSegmentWeights, self).__init__()
    self._indexes_per_weight = {}
    self._weight_per_index = {}

  def AddIndex(self, path_segment_index):
    """Adds a path segment index and sets its weight to 0.

    Args:
      path_segment_index: an integer containing the path segment index.

    Raises:
      ValueError: if the path segment weights already contains
                  the path segment index.
    """
    if path_segment_index in self._weight_per_index:
      raise ValueError(u'Path segment index already set.')

    self._weight_per_index[path_segment_index] = 0

  def AddWeight(self, path_segment_index, weight):
    """Adds a weight for a specific path segment index.

    Args:
      path_segment_index: an integer containing the path segment index.
      weight: an integer containing the weight.

    Raises:
      ValueError: if the path segment weights do not contain
                  the path segment index.
    """
    if path_segment_index not in self._weight_per_index:
      raise ValueError(u'Path segment index not set.')

    self._weight_per_index[path_segment_index] += weight

    if weight not in self._indexes_per_weight:
      self._indexes_per_weight[weight] = []

    self._indexes_per_weight[weight].append(path_segment_index)

  def GetFirstAvailableIndex(self):
    """Retrieves the first available path segment index.

    Returns:
      An integer containing the first available path segment index
      or None if not available.
    """
    path_segment_indexes = sorted(self._weight_per_index.keys())
    if not path_segment_indexes:
      return

    return path_segment_indexes[0]

  def GetLargestWeight(self):
    """Retrieves the largest weight or 0 if none."""
    if not self._indexes_per_weight:
      return 0

    return max(self._indexes_per_weight)

  def GetIndexesForWeight(self, weight):
    """Retrieves the list of path segment indexes for a specific weight."""
    return self._indexes_per_weight[weight]

  def GetWeightForIndex(self, path_segment_index):
    """Retrieves the weight for a specific path segment index."""
    return self._weight_per_index[path_segment_index]

  def SetWeight(self, path_segment_index, weight):
    """Sets a weight for a specific path segment index.

    Args:
      path_segment_index: an integer containing the path segment index.
      weight: an integer containing the weight.

    Raises:
      ValueError: if the path segment weights do not contain
                  the path segment index.
    """
    if path_segment_index not in self._weight_per_index:
      raise ValueError(u'Path segment index not set.')

    self._weight_per_index[path_segment_index] = weight

    if weight not in self._indexes_per_weight:
      self._indexes_per_weight[weight] = []

    self._indexes_per_weight[weight].append(path_segment_index)

  def ToDebugString(self):
    """Converts the path segment weights into a debug string."""
    text_parts = [u'Path segment index\tWeight']
    for path_segment_index, weight in self._weight_per_index.items():
      text_parts.append(u'{0:d}\t\t\t{1:d}'.format(
          path_segment_index, weight))
    text_parts.append(u'')

    text_parts.append(u'Weight\t\t\tPath segment index(es)')
    for weight, path_segment_indexes in self._indexes_per_weight.items():
      text_parts.append(u'{0:d}\t\t\t{1!s}'.format(
          weight, path_segment_indexes))
    text_parts.append(u'')

    return u'\n'.join(text_parts)


class PathFilterScanTree(object):
  """Class that implements a path filter scan tree."""

  def __init__(self, paths, case_sensitive=True, path_segment_separator=u'/'):
    """Initializes and builds a path filter scan tree.

    Args:
      paths: a list of strings containing the paths.
      case_sensitive: optional boolean value to indicate string matches should
                      be case sensitive.
      path_segment_separator: optional string containing the path segment
                              separator.
    """
    super(PathFilterScanTree, self).__init__()
    self._case_sensitive = case_sensitive
    self._path_segment_separator = path_segment_separator
    self._root_node = None

    if not self._case_sensitive:
      paths = [path.lower() for path in paths]

    path_filter_table = _PathFilterTable(
        paths, [], path_segment_separator=self._path_segment_separator)

    if path_filter_table.paths:
      self._root_node = self._BuildScanTreeNode(path_filter_table, [])

  def _BuildScanTreeNode(self, path_filter_table, ignore_list):
    """Builds a scan tree node.

    Args:
      path_filter_table: a path filter table object (instance of
                         _PathFilterTable).
      ignore_list: a list of path segment indexes to ignore, where 0 is the
                   index of the first path segment relative from the root.

    Returns:
      A scan tree node (instance of PathFilterScanTreeNode).

    Raises:
      ValueError: if the path segment index value or the number of paths
                  segments value is out of bounds.
    """
    # Make a copy of the lists because the function is going to alter them
    # and the changes must remain in scope of the function.
    paths_list = list(path_filter_table.paths)
    ignore_list = list(ignore_list)

    similarity_weights = _PathSegmentWeights()
    occurrence_weights = _PathSegmentWeights()
    value_weights = _PathSegmentWeights()

    for path_segment_index in path_filter_table.path_segments_per_index.keys():
      # Skip a path segment index for which no path segments are defined
      # in the path filter table.
      if not path_filter_table.path_segments_per_index[path_segment_index]:
        continue

      similarity_weights.AddIndex(path_segment_index)
      occurrence_weights.AddIndex(path_segment_index)
      value_weights.AddIndex(path_segment_index)

      path_segments = path_filter_table.GetPathSegments(path_segment_index)

      number_of_path_segments = len(path_segments.keys())
      if number_of_path_segments > 1:
        occurrence_weights.SetWeight(
            path_segment_index, number_of_path_segments)

      for paths_per_segment_list in path_segments.values():
        path_segment_weight = len(paths_per_segment_list)
        if path_segment_weight > 1:
          similarity_weights.AddWeight(path_segment_index, path_segment_weight)

    path_segment_index = self._GetMostSignificantPathSegmentIndex(
        paths_list, similarity_weights, occurrence_weights, value_weights)

    ignore_list.append(path_segment_index)

    if path_segment_index < 0:
      raise ValueError(u'Invalid path segment index value out of bounds.')

    scan_tree_node = PathFilterScanTreeNode(path_segment_index)

    path_segments = path_filter_table.GetPathSegments(path_segment_index)

    for path_segment, paths_per_segment_list in path_segments.items():
      if not paths_per_segment_list:
        raise ValueError(u'Invalid number of paths value out of bounds.')

      if len(paths_per_segment_list) == 1:
        for path in paths_per_segment_list:
          scan_tree_node.AddPathSegment(path_segment, path)

      else:
        sub_path_filter_table = _PathFilterTable(
            paths_per_segment_list, ignore_list,
            path_segment_separator=self._path_segment_separator)

        scan_sub_node = self._BuildScanTreeNode(
            sub_path_filter_table, ignore_list)

        scan_tree_node.AddPathSegment(path_segment, scan_sub_node)

      for path in paths_per_segment_list:
        paths_list.remove(path)

    number_of_paths = len(paths_list)
    if number_of_paths == 1:
      scan_tree_node.SetDefaultValue(paths_list[0])

    elif number_of_paths > 1:
      path_filter_table = _PathFilterTable(
          paths_list, ignore_list,
          path_segment_separator=self._path_segment_separator)

      scan_sub_node = self._BuildScanTreeNode(path_filter_table, ignore_list)

      scan_tree_node.SetDefaultValue(scan_sub_node)

    return scan_tree_node

  def _GetMostSignificantPathSegmentIndex(
      self, paths, similarity_weights, occurrence_weights, value_weights):
    """Retrieves the index of the most significant path segment.

    Args:
      paths: a list of strings containing the paths.
      similarity_weights: the similarity weights object (instance of
                          _PathSegmentWeights).
      occurrence_weights: the occurrence weights object (instance of
                          _PathSegmentWeights).
      value_weights: the value weights object (instance of _PathSegmentWeights).

    Returns:
      An integer containing the path segment index.

    Raises:
      ValueError: when paths is an empty list.
    """
    if not paths:
      raise ValueError(u'Missing paths.')

    number_of_paths = len(paths)

    path_segment_index = None
    if number_of_paths == 1:
      path_segment_index = self._GetPathSegmentIndexForValueWeights(
          value_weights)

    elif number_of_paths == 2:
      path_segment_index = self._GetPathSegmentIndexForOccurrenceWeights(
          occurrence_weights, value_weights)

    elif number_of_paths > 2:
      path_segment_index = self._GetPathSegmentIndexForSimilarityWeights(
          similarity_weights, occurrence_weights, value_weights)

    return path_segment_index

  def _GetPathSegmentIndexForOccurrenceWeights(
      self, occurrence_weights, value_weights):
    """Retrieves the index of the path segment based on occurrence weights.

    Args:
      occurrence_weights: the occurrence weights object (instance of
                          _PathSegmentWeights).
      value_weights: the value weights object (instance of _PathSegmentWeights).

    Returns:
      An integer containing the path segment index.
    """
    largest_weight = occurrence_weights.GetLargestWeight()

    if largest_weight > 0:
      occurrence_weight_indexes = occurrence_weights.GetIndexesForWeight(
          largest_weight)
      number_of_occurrence_indexes = len(occurrence_weight_indexes)
    else:
      number_of_occurrence_indexes = 0

    path_segment_index = None
    if number_of_occurrence_indexes == 0:
      path_segment_index = self._GetPathSegmentIndexForValueWeights(
          value_weights)

    elif number_of_occurrence_indexes == 1:
      path_segment_index = occurrence_weight_indexes[0]

    else:
      largest_weight = 0

      for occurrence_index in occurrence_weight_indexes:
        value_weight = value_weights.GetWeightForIndex(occurrence_index)

        if not path_segment_index or largest_weight < value_weight:
          largest_weight = value_weight
          path_segment_index = occurrence_index

    return path_segment_index

  def _GetPathSegmentIndexForSimilarityWeights(
      self, similarity_weights, occurrence_weights, value_weights):
    """Retrieves the index of the path segment based on similarity weights.

    Args:
      similarity_weights: the similarity weights object (instance of
                          _PathSegmentWeights).
      occurrence_weights: the occurrence weights object (instance of
                          _PathSegmentWeights).
      value_weights: the value weights object (instance of _PathSegmentWeights).

    Returns:
      An integer containing the path segment index.
    """
    largest_weight = similarity_weights.GetLargestWeight()

    if largest_weight > 0:
      similarity_weight_indexes = similarity_weights.GetIndexesForWeight(
          largest_weight)
      number_of_similarity_indexes = len(similarity_weight_indexes)
    else:
      number_of_similarity_indexes = 0

    path_segment_index = None
    if number_of_similarity_indexes == 0:
      path_segment_index = self._GetPathSegmentIndexForOccurrenceWeights(
          occurrence_weights, value_weights)

    elif number_of_similarity_indexes == 1:
      path_segment_index = similarity_weight_indexes[0]

    else:
      largest_weight = 0
      largest_value_weight = 0

      for similarity_index in similarity_weight_indexes:
        occurrence_weight = occurrence_weights.GetWeightForIndex(
            similarity_index)

        if largest_weight > 0 and largest_weight == occurrence_weight:
          value_weight = value_weights.GetWeightForIndex(similarity_index)

          if largest_value_weight < value_weight:
            largest_weight = 0

        if not path_segment_index or largest_weight < occurrence_weight:
          largest_weight = occurrence_weight
          path_segment_index = similarity_index

          largest_value_weight = value_weights.GetWeightForIndex(
              similarity_index)

    return path_segment_index

  def _GetPathSegmentIndexForValueWeights(self, value_weights):
    """Retrieves the index of the path segment based on value weights.

    Args:
      value_weights: the value weights object (instance of _PathSegmentWeights).

    Returns:
      An integer containing the path segment index.

    Raises:
      RuntimeError: is no path segment index can be found.
    """
    largest_weight = value_weights.GetLargestWeight()

    if largest_weight > 0:
      value_weight_indexes = value_weights.GetIndexesForWeight(largest_weight)
    else:
      value_weight_indexes = []

    if value_weight_indexes:
      path_segment_index = value_weight_indexes[0]
    else:
      path_segment_index = value_weights.GetFirstAvailableIndex()

    if path_segment_index is None:
      raise RuntimeError(u'No path segment index found.')

    return path_segment_index

  def CheckPath(self, path, path_segment_separator=None):
    """Checks if a path matches the scan tree-based path filter.

    Args:
      path: a string containing the path.
      path_segment_separator: optional string containing the path segment
                              separator. None defaults to the path segment
                              separator that was set when the path filter
                              scan tree was initialized.

    Returns:
      A boolean indicating if the path matches the filter.
    """
    if not self._case_sensitive:
      path = path.lower()

    if path_segment_separator is None:
      path_segment_separator = self._path_segment_separator

    path_segments = path.split(path_segment_separator)
    number_of_path_segments = len(path_segments)

    scan_object = self._root_node
    while scan_object:
      if isinstance(scan_object, py2to3.STRING_TYPES):
        break

      if scan_object.path_segment_index >= number_of_path_segments:
        scan_object = scan_object.default_value
        continue

      path_segment = path_segments[scan_object.path_segment_index]
      scan_object = scan_object.GetScanObject(path_segment)

    if not isinstance(scan_object, py2to3.STRING_TYPES):
      return False

    filter_path_segments = scan_object.split(self._path_segment_separator)
    return filter_path_segments == path_segments


class PathFilterScanTreeNode(object):
  """Class that implements a path filter scan tree node.

  The path filter scan tree node defines the path segments for a specific
  path segment index to filter. Each path segment will point to a scan object
  that indicates the next part of the path filter. A default value indicates
  the scan object to use next when there was no match.

  Attributes:
    default_value: the default scan object, either a scan tree sub node
                   (instance of PathFilterScanTreeNode) or a string
                   containing a path.
    parent: the parent path filter scan tree node or None.
    path_segment_index: an integer containing the path segment index
                        represented by the node.
  """

  def __init__(self, path_segment_index):
    """Initializes a path filter scan tree node.

    Args:
      path_segment_index: an integer containing the path segment index.
    """
    super(PathFilterScanTreeNode, self).__init__()
    self._path_segments = {}
    self.default_value = None
    self.parent = None
    self.path_segment_index = path_segment_index

  @property
  def path_segments(self):
    """A list of strings containing the path segments."""
    return self._path_segments.keys()

  def AddPathSegment(self, path_segment, scan_object):
    """Adds a path segment.

    Args:
      path_segment: a string containing the path segment.
      scan_object: a scan object, either a scan tree sub node (instance of
                   PathFilterScanTreeNode) or a string containing a path.

    Raises:
      ValueError: if the node already contains a scan object for
                  the path segment.
    """
    if path_segment in self._path_segments:
      raise ValueError(u'Path segment already set.')

    if isinstance(scan_object, PathFilterScanTreeNode):
      scan_object.parent = self

    self._path_segments[path_segment] = scan_object

  def GetScanObject(self, path_segment):
    """Retrieves the scan object for a specific path segment.

    Args:
      path_segment: a string containing the path segment.

    Returns:
      A scan object, which can be a scan tree sub node (instance of
      PathFilterScanTreeNode), a path or the default value.
    """
    return self._path_segments.get(path_segment, self.default_value)

  def SetDefaultValue(self, scan_object):
    """Sets the default (non-match) value.

    Args:
      scan_object: a scan object, either a scan tree sub node (instance of
                   PathFilterScanTreeNode) or a string containing a path.

    Raises:
      TypeError: if the scan object is of an unsupported type.
      ValueError: if the default value is already set.
    """
    if (not isinstance(scan_object, PathFilterScanTreeNode) and
        not isinstance(scan_object, py2to3.STRING_TYPES)):
      raise TypeError(u'Unsupported scan object type.')

    if self.default_value:
      raise ValueError(u'Default value already set.')

    self.default_value = scan_object

  def ToDebugString(self, indentation_level=1):
    """Converts the path filter scan tree node into a debug string.

    Args:
      indentation_level: an integer containing the text identation level.

    Returns:
      A string containing a debug representation of the path filter scan
      tree node.
    """
    indentation = u'  ' * indentation_level

    text_parts = [u'{0:s}path segment index: {1:d}\n'.format(
        indentation, self.path_segment_index)]

    for path_segment, scan_object in self._path_segments.items():
      text_parts.append(u'{0:s}path segment: {1:s}\n'.format(
          indentation, path_segment))

      if isinstance(scan_object, PathFilterScanTreeNode):
        text_parts.append(u'{0:s}scan tree node:\n'.format(indentation))
        text_parts.append(scan_object.ToDebugString(indentation_level + 1))

      elif isinstance(scan_object, py2to3.STRING_TYPES):
        text_parts.append(u'{0:s}path: {1:s}\n'.format(
            indentation, scan_object))

    text_parts.append(u'{0:s}default value:\n'.format(indentation))

    if isinstance(self.default_value, PathFilterScanTreeNode):
      text_parts.append(u'{0:s}scan tree node:\n'.format(indentation))
      text_parts.append(self.default_value.ToDebugString(indentation_level + 1))

    elif isinstance(self.default_value, py2to3.STRING_TYPES):
      text_parts.append(u'{0:s}pattern: {1:s}\n'.format(
          indentation, self.default_value))

    text_parts.append(u'\n')

    return u''.join(text_parts)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the syslog parser."""

import unittest

from plaso.formatters import syslog  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers import syslog

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class SyslogParserTest(test_lib.ParserTestCase):
  """Tests for the syslog parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'syslog_rsyslog'])
  def testParseRsyslog(self):
    """Tests the Parse function on an Ubuntu-style syslog file"""
    parser = syslog.SyslogParser()
    knowledge_base_values = {u'year': 2016}
    storage_writer = self._ParseFile(
        [u'syslog_rsyslog'], parser,
        knowledge_base_values=knowledge_base_values)

    self.assertEqual(storage_writer.number_of_events, 8)

  @shared_test_lib.skipUnlessHasTestFile([u'syslog_osx'])
  def testParseDarwin(self):
    """Tests the Parse function on an Darwin-style syslog file"""
    parser = syslog.SyslogParser()
    knowledge_base_values = {u'year': 2016}
    storage_writer = self._ParseFile(
        [u'syslog_osx'], parser,
        knowledge_base_values=knowledge_base_values)

    self.assertEqual(storage_writer.number_of_events, 2)

  @shared_test_lib.skipUnlessHasTestFile([u'syslog_chromeos'])
  def testParseChromeOS(self):
    """Tests the Parse function."""
    parser = syslog.SyslogParser()
    knowledge_base_values = {u'year': 2016}
    storage_writer = self._ParseFile(
        [u'syslog_chromeos'], parser,
        knowledge_base_values=knowledge_base_values)

    self.assertEqual(storage_writer.number_of_events, 8)

    events = list(storage_writer.GetEvents())

    event = events[0]
    event_timestamp = timelib.Timestamp.CopyToIsoFormat(
        event.timestamp)
    self.assertEqual(event_timestamp, u'2016-10-25T19:37:23.297265+00:00')

    expected_message = (
        u'INFO [periodic_scheduler, pid: 13707] cleanup_logs: job completed')
    self._TestGetMessageStrings(event, expected_message, expected_message)

    event = events[2]
    event_timestamp = timelib.Timestamp.CopyToIsoFormat(
        event.timestamp)
    self.assertEqual(event_timestamp, u'2016-10-25T19:37:24.987014+00:00')

    # Testing year increment.
    event = events[4]
    event_timestamp = timelib.Timestamp.CopyToIsoFormat(
        event.timestamp)
    self.assertEqual(event_timestamp, u'2016-10-25T19:37:24.993079+00:00')

    event = events[6]
    expected_reporter = u'kernel'
    self.assertEqual(event.reporter, expected_reporter)

    event = events[7]
    expected_message = (
        u'INFO [aprocess] [  316.587330] cfg80211: This is a multi-line\t'
        u'message that screws up many syslog parsers.')
    expected_short_message = (
        u'INFO [aprocess] [  316.587330] cfg80211: This is a multi-line\t'
        u'message that sc...')
    self._TestGetMessageStrings(event, expected_message, expected_short_message)

  @shared_test_lib.skipUnlessHasTestFile([u'syslog'])
  def testParse(self):
    """Tests the Parse function."""
    parser = syslog.SyslogParser()
    knowledge_base_values = {u'year': 2012}
    storage_writer = self._ParseFile(
        [u'syslog'], parser,
        knowledge_base_values=knowledge_base_values)

    self.assertEqual(storage_writer.number_of_events, 16)

    events = list(storage_writer.GetEvents())

    event = events[0]
    event_timestamp = timelib.Timestamp.CopyToIsoFormat(
        event.timestamp)
    self.assertEqual(event_timestamp, u'2012-01-22T07:52:33+00:00')
    self.assertEqual(event.hostname, u'myhostname.myhost.com')

    expected_message = (
        u'[client, pid: 30840] INFO No new content in ímynd.dd.')
    self._TestGetMessageStrings(event, expected_message, expected_message)

    event = events[6]
    event_timestamp = timelib.Timestamp.CopyToIsoFormat(
        event.timestamp)
    self.assertEqual(event_timestamp, u'2012-02-29T01:15:43+00:00')

    # Testing year increment.
    event = events[8]
    event_timestamp = timelib.Timestamp.CopyToIsoFormat(
        event.timestamp)
    self.assertEqual(event_timestamp, u'2013-03-23T23:01:18+00:00')

    event = events[10]
    expected_reporter = u'/sbin/anacron'
    self.assertEqual(event.reporter, expected_reporter)

    event = events[11]
    expected_message = (
        u'[aprocess, pid: 10100] This is a multi-line message that screws up'
        u'\tmany syslog parsers.')
    expected_short_message = (
        u'[aprocess, pid: 10100] This is a multi-line message that screws up'
        u'\tmany syslo...')
    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    event = events[14]
    self.assertEqual(event.reporter, u'kernel')
    self.assertIsNone(event.hostname)
    expected_message = (
        u'[kernel] [997.390602] sda2: rw=0, want=65, limit=2')
    expected_short_message = (
        u'[kernel] [997.390602] sda2: rw=0, want=65, limit=2')
    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    # Testing non-leap year.
    parser = syslog.SyslogParser()
    knowledge_base_values = {u'year': 2013}
    storage_writer = self._ParseFile(
        [u'syslog'], parser,
        knowledge_base_values=knowledge_base_values)

    self.assertEqual(storage_writer.number_of_events, 15)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Mac OS X application usage database plugin."""

import unittest

from plaso.formatters import appusage  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers.sqlite_plugins import appusage

from tests import test_lib as shared_test_lib
from tests.parsers.sqlite_plugins import test_lib


class ApplicationUsagePluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the Mac OS X application usage activity database plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'application_usage.sqlite'])
  def testProcess(self):
    """Tests the Process function."""
    plugin = appusage.ApplicationUsagePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        [u'application_usage.sqlite'], plugin)

    # The sqlite database contains 5 events.
    self.assertEqual(storage_writer.number_of_events, 5)

    events = list(storage_writer.GetEvents())

    # Check the first event.
    event = events[0]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-05-07 18:52:02')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.application, u'/Applications/Safari.app')
    self.assertEqual(event.app_version, u'9537.75.14')
    self.assertEqual(event.bundle_id, u'com.apple.Safari')
    self.assertEqual(event.count, 1)

    expected_message = (
        u'/Applications/Safari.app v.9537.75.14 '
        u'(bundle: com.apple.Safari). '
        u'Launched: 1 time(s)')

    expected_short_message = u'/Applications/Safari.app (1 time(s))'

    self._TestGetMessageStrings(event, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The Windows Scheduled Task (job) event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager
from plaso.lib import errors


class WinJobFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Windows Scheduled Task (job) event."""

  DATA_TYPE = u'windows:tasks:job'

  FORMAT_STRING_PIECES = [
      u'Application: {application}',
      u'{parameters}',
      u'Scheduled by: {username}',
      u'Working directory: {working_directory}',
      u'Trigger type: {trigger_type}']

  SOURCE_LONG = u'Windows Scheduled Task Job'
  SOURCE_SHORT = u'JOB'

  _TRIGGER_TYPES = {
      0x0000: u'ONCE',
      0x0001: u'DAILY',
      0x0002: u'WEEKLY',
      0x0003: u'MONTHLYDATE',
      0x0004: u'MONTHLYDOW',
      0x0005: u'EVENT_ON_IDLE',
      0x0006: u'EVENT_AT_SYSTEMSTART',
      0x0007: u'EVENT_AT_LOGON'
  }

  def GetMessages(self, unused_formatter_mediator, event):
    """Determines the formatted message strings for an event object.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event (EventObject): event.

    Returns:
      tuple(str, str): formatted message string and short message string.

    Raises:
      WrongFormatter: if the event object cannot be formatted by the formatter.
    """
    if self.DATA_TYPE != event.data_type:
      raise errors.WrongFormatter(u'Unsupported data type: {0:s}.'.format(
          event.data_type))

    event_values = event.CopyToDict()

    trigger_type = event_values.get(u'trigger_type', None)
    if trigger_type is not None:
      event_values[u'trigger_type'] = self._TRIGGER_TYPES.get(
          trigger_type, u'0x{0:04x}'.format(trigger_type))

    return self._ConditionalFormatMessages(event_values)


manager.FormattersManager.RegisterFormatter(WinJobFormatter)



# -*- coding: utf-8 -*-
"""The Apple System Log (ASL) event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager
from plaso.lib import errors
from plaso.lib import py2to3


class ASLFormatter(interface.ConditionalEventFormatter):
  """Formatter for an Apple System Log (ASL) log event."""

  DATA_TYPE = u'mac:asl:event'

  FORMAT_STRING_PIECES = [
      u'MessageID: {message_id}',
      u'Level: {level}',
      u'User ID: {user_sid}',
      u'Group ID: {group_id}',
      u'Read User: {read_uid}',
      u'Read Group: {read_gid}',
      u'Host: {computer_name}',
      u'Sender: {sender}',
      u'Facility: {facility}',
      u'Message: {message}',
      u'{extra_information}']

  FORMAT_STRING_SHORT_PIECES = [
      u'Host: {host}',
      u'Sender: {sender}',
      u'Facility: {facility}']

  SOURCE_LONG = u'ASL entry'
  SOURCE_SHORT = u'LOG'

  # Priority levels (criticality)
  _PRIORITY_LEVELS = {
      0 : u'EMERGENCY',
      1 : u'ALERT',
      2 : u'CRITICAL',
      3 : u'ERROR',
      4 : u'WARNING',
      5 : u'NOTICE',
      6 : u'INFO',
      7 : u'DEBUG'}

  def GetMessages(self, unused_formatter_mediator, event):
    """Determines the formatted message strings for an event object.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event (EventObject): event.

    Returns:
      tuple(str, str): formatted message string and short message string.

    Raises:
      WrongFormatter: if the event object cannot be formatted by the formatter.
    """
    if self.DATA_TYPE != event.data_type:
      raise errors.WrongFormatter(u'Unsupported data type: {0:s}.'.format(
          event.data_type))

    event_values = event.CopyToDict()

    priority_level = event_values.get(u'level', None)
    if isinstance(priority_level, py2to3.INTEGER_TYPES):
      event_values[u'level'] = u'{0:s} ({1:d})'.format(
          self._PRIORITY_LEVELS.get(priority_level, u'UNKNOWN'), priority_level)

    # If no rights are assigned the value is 0xffffffff (-1).
    read_uid = event_values.get(u'read_uid', None)
    if read_uid == 0xffffffff:
      event_values[u'read_uid'] = u'ALL'

    # If no rights are assigned the value is 0xffffffff (-1).
    read_gid = event_values.get(u'read_gid', None)
    if read_gid == 0xffffffff:
      event_values[u'read_gid'] = u'ALL'

    # TODO: get the real name for the user of the group having the uid or gid.
    return self._ConditionalFormatMessages(event_values)


manager.FormattersManager.RegisterFormatter(ASLFormatter)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Safari history plist plugin."""

import unittest

from plaso.formatters import plist  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers.plist_plugins import safari

from tests import test_lib as shared_test_lib
from tests.parsers.plist_plugins import test_lib


class SafariPluginTest(test_lib.PlistPluginTestCase):
  """Tests for the Safari history plist plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'History.plist'])
  def testProcess(self):
    """Tests the Process function."""
    plist_name = u'History.plist'

    plugin = safari.SafariHistoryPlugin()
    storage_writer = self._ParsePlistFileWithPlugin(
        plugin, [plist_name], plist_name)

    self.assertEqual(storage_writer.number_of_events, 18)

    # The order in which PlistParser generates events is nondeterministic
    # hence we sort the events.
    events = list(storage_writer.GetSortedEvents())

    event = events[7]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-07-08 17:31:00')
    self.assertEqual(event.timestamp, expected_timestamp)

    event = events[9]

    expected_url = u'http://netverslun.sci-mx.is/aminosyrur'
    self.assertEqual(event.url, expected_url)

    expected_message = (
        u'Visited: {0:s} (Am\xedn\xf3s\xfdrur ) '
        u'Visit Count: 1').format(expected_url)

    self._TestGetMessageStrings(event, expected_message, expected_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains preprocessors for Linux."""

from __future__ import unicode_literals

import csv
import datetime
import logging

from dateutil import tz

from dfvfs.helpers import text_file as dfvfs_text_file

from plaso.containers import artifacts
from plaso.lib import errors
from plaso.preprocessors import interface
from plaso.preprocessors import manager


class LinuxHostnamePlugin(interface.FileArtifactPreprocessorPlugin):
  """The Linux hostname plugin."""

  ARTIFACT_DEFINITION_NAME = 'LinuxHostnameFile'

  def _ParseFileData(self, knowledge_base, file_object):
    """Parses file content (data) for a hostname preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      file_object (dfvfs.FileIO): file-like object that contains the artifact
          value data.

    Returns:
      bool: True if all the preprocessing attributes were found and
          the preprocessor plugin is done.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    result = False
    text_file_object = dfvfs_text_file.TextFile(file_object)
    hostname = text_file_object.readline()

    try:
      hostname = hostname.decode('utf-8')
    except UnicodeDecodeError:
      # TODO: add and store preprocessing errors.
      hostname = hostname.decode('utf-8', errors='replace')

    hostname = hostname.strip()
    if hostname:
      hostname_artifact = artifacts.HostnameArtifact(name=hostname)
      knowledge_base.SetHostname(hostname_artifact)
      result = True

    return result


class LinuxTimeZonePlugin(interface.FileEntryArtifactPreprocessorPlugin):
  """Linux time zone plugin."""

  ARTIFACT_DEFINITION_NAME = 'LinuxLocalTime'

  def _ParseFileEntry(self, knowledge_base, file_entry):
    """Parses artifact file system data for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      file_entry (dfvfs.FileEntry): file entry that contains the artifact
          value data.

    Returns:
      bool: True if all the preprocessing attributes were found and
          the preprocessor plugin is done.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    result = False

    if file_entry.link:
      # Determine the timezone based on the file path.
      _, _, time_zone = file_entry.link.partition('zoneinfo/')

    else:
      # Determine the timezone based on the timezone information file.
      file_object = file_entry.GetFileObject()

      time_zone = None
      try:
        time_zone_file = tz.tzfile(file_object)
        date_time = datetime.datetime(2017, 1, 1)
        time_zone = time_zone_file.tzname(date_time)

      except ValueError:
        # TODO: add and store preprocessing errors.
        logging.error('Unable to read time zone information file.')

      finally:
        file_object.close()

    if time_zone:
      try:
        knowledge_base.SetTimeZone(time_zone)
        result = True
      except ValueError:
        # TODO: add and store preprocessing errors.
        logging.error('Unable to set time zone in knowledge base.')

    return result


class LinuxUserAccountsPlugin(interface.FileArtifactPreprocessorPlugin):
  """The Linux user accounts plugin."""

  ARTIFACT_DEFINITION_NAME = 'LinuxPasswdFile'

  def _ParseFileData(self, knowledge_base, file_object):
    """Parses file content (data) for user account preprocessing attributes.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      file_object (dfvfs.FileIO): file-like object that contains the artifact
          value data.

    Returns:
      bool: True if all the preprocessing attributes were found and
          the preprocessor plugin is done.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    result = False
    text_file_object = dfvfs_text_file.TextFile(file_object)

    try:
      reader = csv.reader(text_file_object, delimiter=b':')
    except csv.Error as exception:
      raise errors.PreProcessFail(
          'Unable to read: {0:s} with error: {1!s}'.format(
              self.ARTIFACT_DEFINITION_NAME, exception))

    for row in reader:
      if len(row) < 7 or not row[0] or not row[2]:
        # TODO: add and store preprocessing errors.
        continue

      user_account = artifacts.UserAccountArtifact(
          identifier=row[2], username=row[0])
      user_account.group_identifier = row[3] or None
      user_account.full_name = row[4] or None
      user_account.user_directory = row[5] or None
      user_account.shell = row[6] or None

      try:
        knowledge_base.AddUserAccount(user_account)
        result = True
      except KeyError:
        # TODO: add and store preprocessing errors.
        pass

    return result


manager.PreprocessPluginsManager.RegisterPlugins([
    LinuxHostnamePlugin, LinuxTimeZonePlugin, LinuxUserAccountsPlugin])



# -*- coding: utf-8 -*-
"""Analysis plugin to look up files in nsrlsvr and tag events."""

import logging
import socket

from plaso.analysis import interface
from plaso.analysis import manager


class NsrlsvrAnalyzer(interface.HashAnalyzer):
  """Analyzes file hashes by consulting an nsrlsvr instance.

  Attributes:
    analyses_performed (int): number of analysis batches completed by this
        analyzer.
    hashes_per_batch (int): maximum number of hashes to analyze at once.
    seconds_spent_analyzing (int): number of seconds this analyzer has spent
        performing analysis (as opposed to waiting on queues, etc.)
    wait_after_analysis (int): number of seconds the analyzer will sleep for
        after analyzing a batch of hashes.
  """
  _RECEIVE_BUFFER_SIZE = 4096
  _SOCKET_TIMEOUT = 3

  SUPPORTED_HASHES = [u'md5', u'sha1']

  def __init__(self, hash_queue, hash_analysis_queue, **kwargs):
    """Initializes an nsrlsvr analyzer thread.

    Args:
      hash_queue (Queue.queue): contains hashes to be analyzed.
      hash_analysis_queue (Queue.queue): that the analyzer will append
          HashAnalysis objects this queue.
    """
    super(NsrlsvrAnalyzer, self).__init__(
        hash_queue, hash_analysis_queue, **kwargs)
    self._host = None
    self._port = None
    self.hashes_per_batch = 100

  def _GetSocket(self):
    """Establishes a connection to an nsrlsvr instance.

    Returns:
      socket._socketobject: socket connected to an nsrlsvr instance or None if
          a connection cannot be established.
    """
    try:
      return socket.create_connection(
          (self._host, self._port), self._SOCKET_TIMEOUT)

    except socket.error as exception:
      logging.error(
          u'Unable to connect to nsrlsvr with error: {0:s}.'.format(exception))

  def _QueryHash(self, nsrl_socket, digest):
    """Queries nsrlsvr for a specfic hash.

    Args:
      nsrl_socket (socket._socketobject): socket of connection to nsrlsvr.
      digest (str): hash to look up.

    Returns:
      bool: True if the hash was found, False if not or None on error.
    """
    query = u'QUERY {0:s}\n'.format(digest)
    response = None

    try:
      nsrl_socket.sendall(query)
      response = nsrl_socket.recv(self._RECEIVE_BUFFER_SIZE)

    except socket.error as exception:
      logging.error(
          u'Unable to query nsrlsvr with error: {0:s}.'.format(exception))

    if not response:
      return False

    # Strip end-of-line characters since they can differ per platform on which
    # nsrlsvr is running.
    response = response.strip()
    # nsrlsvr returns "OK 1" if the has was found or "OK 0" if not.
    return response == b'OK 1'

  def Analyze(self, hashes):
    """Looks up hashes in nsrlsvr.

    Args:
      hashes (list[str]): hash values to look up.

    Returns:
      list[HashAnalysis]: analysis results, or an empty list on error.
    """
    logging.debug(
        u'Opening connection to {0:s}:{1:d}'.format(self._host, self._port))

    nsrl_socket = self._GetSocket()
    if not nsrl_socket:
      self.SignalAbort()
      return []

    hash_analyses = []
    for digest in hashes:
      response = self._QueryHash(nsrl_socket, digest)
      if response is None:
        continue

      hash_analysis = interface.HashAnalysis(digest, response)
      hash_analyses.append(hash_analysis)

    nsrl_socket.close()

    logging.debug(
        u'Closed connection to {0:s}:{1:d}'.format(self._host, self._port))

    return hash_analyses

  def SetHost(self, host):
    """Sets the address or hostname of the server running nsrlsvr.

    Args:
      host (str): IP address or hostname to query.
    """
    self._host = host

  def SetPort(self, port):
    """Sets the port where nsrlsvr is listening.

    Args:
      port (int): port to query.
    """
    self._port = port

  def TestConnection(self):
    """Tests the connection to nsrlsvr.

    Checks if a connection can be set up and queries the server for the
    MD5 of an empty file and expects a response. The value of the response
    is not checked.

    Returns:
      bool: True if nsrlsvr instance is reachable.
    """
    response = None
    nsrl_socket = self._GetSocket()
    if nsrl_socket:
      response = self._QueryHash(
          nsrl_socket, u'd41d8cd98f00b204e9800998ecf8427e')
      nsrl_socket.close()

    return response is not None


class NsrlsvrAnalysisPlugin(interface.HashTaggingAnalysisPlugin):
  """An analysis plugin for looking up hashes in nsrlsvr."""

  # The NSRL contains files of all different types, and can handle a high load
  # so look up all files.
  DATA_TYPES = [u'fs:stat', u'fs:stat:ntfs']

  URLS = [u'https://rjhansen.github.io/nsrlsvr/']

  NAME = u'nsrlsvr'

  def __init__(self):
    """Initializes an nsrlsvr analysis plugin."""
    super(NsrlsvrAnalysisPlugin, self).__init__(NsrlsvrAnalyzer)
    self._label = None

  def GenerateLabels(self, hash_information):
    """Generates a list of strings that will be used in the event tag.

    Args:
      hash_information (bool): whether the analyzer received a response from
          nsrlsvr indicating that the hash was present in its loaded NSRL
          set.

    Returns:
      list[str]: strings describing the results from nsrlsvr.
    """
    if hash_information:
      return [self._label]
    # TODO: Renable when tagging is removed from the analysis report.
    # return [u'nsrl_not_present']
    return []

  def SetLabel(self, label):
    """

    Args:
      label (str): label to apply to events extracted from files that are
          present in nsrlsvr.
    """
    self._label = label

  def SetHost(self, host):
    """Sets the address or hostname of the server running nsrlsvr.

    Args:
      host (str): IP address or hostname to query.
    """
    self._analyzer.SetHost(host)

  def SetPort(self, port):
    """Sets the port where nsrlsvr is listening.

    Args:
      port (int): port to query.
    """
    self._analyzer.SetPort(port)

  def TestConnection(self):
    """Tests the connection to nsrlsvr.

    Returns:
      bool: True if nsrlsvr instance is reachable.
    """
    return self._analyzer.TestConnection()


manager.AnalysisPluginManager.RegisterPlugin(NsrlsvrAnalysisPlugin)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests the plist parser."""

import unittest

from plaso.parsers import plist
# Register all plugins.
from plaso.parsers import plist_plugins  # pylint: disable=unused-import

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class PlistParserTest(test_lib.ParserTestCase):
  """Tests the plist parser."""

  # pylint: disable=protected-access

  def testEnablePlugins(self):
    """Tests the EnablePlugins function."""
    parser = plist.PlistParser()
    parser.EnablePlugins([u'airport'])

    self.assertIsNotNone(parser)
    self.assertIsNotNone(parser._default_plugin)
    self.assertNotEqual(parser._plugins, [])
    self.assertEqual(len(parser._plugins), 1)

  @shared_test_lib.skipUnlessHasTestFile([u'plist_binary'])
  def testParse(self):
    """Tests the Parse function."""
    parser = plist.PlistParser()
    storage_writer = self._ParseFile([u'plist_binary'], parser)

    self.assertEqual(storage_writer.number_of_events, 12)

    timestamps, roots, keys = zip(
        *[(event.timestamp, event.root, event.key)
          for event in storage_writer.GetEvents()])

    expected_timestamps = frozenset([
        1345251192528750, 1351827808261762, 1345251268370453,
        1351818803000000, 1351819298997672, 1351818797324095,
        1301012201414766, 1302199013524275, 1341957900020116,
        1350666391557044, 1350666385239661, 1341957896010535])

    self.assertTrue(set(expected_timestamps) == set(timestamps))
    self.assertEqual(12, len(set(timestamps)))

    expected_roots = frozenset([
        u'/DeviceCache/00-0d-fd-00-00-00',
        u'/DeviceCache/44-00-00-00-00-00',
        u'/DeviceCache/44-00-00-00-00-01',
        u'/DeviceCache/44-00-00-00-00-02',
        u'/DeviceCache/44-00-00-00-00-03',
        u'/DeviceCache/44-00-00-00-00-04'])
    self.assertTrue(expected_roots == set(roots))
    self.assertEqual(6, len(set(roots)))

    expected_keys = frozenset([
        u'LastInquiryUpdate',
        u'LastServicesUpdate',
        u'LastNameUpdate'])
    self.assertTrue(expected_keys == set(keys))
    self.assertEqual(3, len(set(keys)))


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The definitions."""

from __future__ import unicode_literals


COMPRESSION_FORMAT_NONE = 'none'
COMPRESSION_FORMAT_ZLIB = 'zlib'

COMPRESSION_FORMATS = frozenset([
    COMPRESSION_FORMAT_NONE,
    COMPRESSION_FORMAT_ZLIB])

DEFAULT_WORKER_MEMORY_LIMIT = 2048 * 1024 * 1024

FAILURE_MODE_EXHAUST_MEMORY = 'exhaust_memory'
FAILURE_MODE_NOT_RESPONDING = 'not_responding'
FAILURE_MODE_TERMINATED = 'terminated'
FAILURE_MODE_TIME_OUT = 'time_out'

OPERATING_SYSTEM_LINUX = 'Linux'
OPERATING_SYSTEM_MACOSX = 'MacOSX'
OPERATING_SYSTEM_UNKNOWN = 'Unknown'
OPERATING_SYSTEM_WINDOWS = 'Windows'

OPERATING_SYSTEMS = frozenset([
    OPERATING_SYSTEM_LINUX,
    OPERATING_SYSTEM_MACOSX,
    OPERATING_SYSTEM_UNKNOWN,
    OPERATING_SYSTEM_WINDOWS])

PROCESSING_STATUS_ABORTED = 'aborted'
PROCESSING_STATUS_ANALYZING = 'analyzing'
PROCESSING_STATUS_COLLECTING = 'collecting'
PROCESSING_STATUS_COMPLETED = 'completed'
PROCESSING_STATUS_ERROR = 'error'
PROCESSING_STATUS_EXPORTING = 'exporting'
PROCESSING_STATUS_EXTRACTING = 'extracting'
PROCESSING_STATUS_FINALIZING = 'finalizing'
PROCESSING_STATUS_HASHING = 'hashing'
PROCESSING_STATUS_IDLE = 'idle'
PROCESSING_STATUS_INITIALIZED = 'initialized'
PROCESSING_STATUS_KILLED = 'killed'
PROCESSING_STATUS_MERGING = 'merging'
PROCESSING_STATUS_NOT_RESPONDING = 'not responding'
PROCESSING_STATUS_REPORTING = 'reporting'
PROCESSING_STATUS_RUNNING = 'running'
PROCESSING_STATUS_YARA_SCAN = 'yara scan'

PROCESSING_ERROR_STATUS = frozenset([
    PROCESSING_STATUS_ABORTED,
    PROCESSING_STATUS_ERROR,
    PROCESSING_STATUS_NOT_RESPONDING,
    PROCESSING_STATUS_KILLED])

RESERVED_VARIABLE_NAMES = frozenset([
    'body',
    'data_type',
    'display_name',
    'filename',
    'hostname',
    'http_headers',
    'inode',
    'mapped_files',
    'metadata',
    'offset',
    'parser',
    'pathspec',
    'query',
    'regvalue',
    'source_long',
    'source_short',
    'tag',
    'text_prepend',
    'timestamp',
    'timestamp_desc',
    'timezone',
    'username'])

SERIALIZER_FORMAT_JSON = 'json'

SERIALIZER_FORMATS = frozenset([SERIALIZER_FORMAT_JSON])

STORAGE_FORMAT_SQLITE = 'sqlite'
STORAGE_FORMAT_ZIP = 'zip'

STORAGE_FORMATS = frozenset([STORAGE_FORMAT_SQLITE, STORAGE_FORMAT_ZIP])

# The session storage contains the results of one or more sessions.
# A typical session is e.g. a single run of a tool (log2timeline.py).
# The task storage contains the results of one or more tasks. Tasks
# are used to split work within a session. A typical task is e.g.
# a single run of a worker process.
STORAGE_TYPE_SESSION = 'session'
STORAGE_TYPE_TASK = 'task'

STORAGE_TYPES = frozenset([STORAGE_TYPE_SESSION, STORAGE_TYPE_TASK])

TIME_DESCRIPTION_ACCOUNT_CREATED = 'Account Created'
TIME_DESCRIPTION_ADDED = 'Creation Time'
TIME_DESCRIPTION_BACKUP = 'Backup Time'
TIME_DESCRIPTION_CHANGE = 'Metadata Modification Time'
TIME_DESCRIPTION_CONNECTION_ESTABLISHED = 'Connection Established'
TIME_DESCRIPTION_CONNECTION_FAILED = 'Connection Failed'
TIME_DESCRIPTION_CREATION = TIME_DESCRIPTION_ADDED
TIME_DESCRIPTION_DELETED = 'Content Deletion Time'
TIME_DESCRIPTION_END = 'End Time'
TIME_DESCRIPTION_ENTRY_MODIFICATION = TIME_DESCRIPTION_CHANGE
TIME_DESCRIPTION_EXIT = 'Exit Time'
TIME_DESCRIPTION_EXPIRATION = 'Expiration Time'
TIME_DESCRIPTION_FILE_DOWNLOADED = 'File Downloaded'
TIME_DESCRIPTION_FIRST_CONNECTED = 'First Connection Time'
TIME_DESCRIPTION_INSTALLATION = 'Installation Time'
TIME_DESCRIPTION_LAST_ACCESS = 'Last Access Time'
TIME_DESCRIPTION_LAST_CHECKED = 'Last Checked Time'
TIME_DESCRIPTION_LAST_CONNECTED = 'Last Connection Time'
TIME_DESCRIPTION_LAST_LOGIN = 'Last Login Time'
TIME_DESCRIPTION_LAST_PASSWORD_RESET = 'Last Password Reset'
TIME_DESCRIPTION_LAST_PRINTED = 'Last Printed Time'
TIME_DESCRIPTION_LAST_RESUME = 'Last Resume Time'
TIME_DESCRIPTION_LAST_RUN = 'Last Time Executed'
TIME_DESCRIPTION_LAST_SHUTDOWN = 'Last Shutdown Time'
TIME_DESCRIPTION_LAST_VISITED = 'Last Visited Time'
TIME_DESCRIPTION_MODIFICATION = 'Content Modification Time'
TIME_DESCRIPTION_START = 'Start Time'
TIME_DESCRIPTION_UPDATE = 'Update Time'
TIME_DESCRIPTION_WRITTEN = TIME_DESCRIPTION_MODIFICATION

# The timestamp does not represent a date and time value.
TIME_DESCRIPTION_NOT_A_TIME = 'Not a time'

# Note that the unknown time is used for date and time values
# of which the exact meaning is unknown and being researched.
# For most cases do not use this timestamp description.
TIME_DESCRIPTION_UNKNOWN = 'Unknown Time'



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Windows Services analysis plugin CLI arguments helper."""

import argparse
import unittest

from plaso.analysis import windows_services
from plaso.lib import errors
from plaso.cli.helpers import windows_services_analysis as arguments_helper

from tests.cli import test_lib as cli_test_lib
from tests.cli.helpers import test_lib


class WindowsServicesAnalysisArgumentsHelperTest(
    test_lib.AnalysisPluginArgumentsHelperTest):
  """Tests the Windows Services analysis plugin CLI arguments helper."""

  _EXPECTED_OUTPUT = u'\n'.join([
      u'usage: cli_helper.py [--windows-services-output {text,yaml}]',
      u'',
      u'Test argument parser.',
      u'',
      u'optional arguments:',
      u'  --windows-services-output {text,yaml}',
      (u'                        Specify how the results should be displayed. '
       u'Options'),
      u'                        are text and yaml.',
      u''])

  def testAddArguments(self):
    """Tests the AddArguments function."""
    argument_parser = argparse.ArgumentParser(
        prog=u'cli_helper.py',
        description=u'Test argument parser.', add_help=False,
        formatter_class=cli_test_lib.SortedArgumentsHelpFormatter)

    arguments_helper.WindowsServicesAnalysisArgumentsHelper.AddArguments(
        argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT)

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    options = cli_test_lib.TestOptions()

    analysis_plugin = windows_services.WindowsServicesAnalysisPlugin()
    arguments_helper.WindowsServicesAnalysisArgumentsHelper.ParseOptions(
        options, analysis_plugin)

    with self.assertRaises(errors.BadConfigObject):
      arguments_helper.WindowsServicesAnalysisArgumentsHelper.ParseOptions(
          options, None)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The analysis plugin mediator object."""

from plaso.engine import path_helper
from plaso.lib import timelib


class AnalysisMediator(object):
  """Class that implements the analysis plugin mediator.

  Attributes:
    number_of_produced_analysis_reports (int): number of produced analysis
        reports.
    number_of_produced_event_tags (int): number of produced event tags.
  """

  def __init__(self, storage_writer, knowledge_base, data_location=None):
    """Initializes an analysis plugin mediator object.

    Args:
      storage_writer (StorageWriter): storage writer.
      knowledge_base (KnowledgeBase): contains information from the source
          data needed for analysis.
      data_location (Optional[str]): location of data files used during
          analysis.
    """
    super(AnalysisMediator, self).__init__()
    self._abort = False
    self._data_location = data_location
    self._event_filter_expression = None
    self._knowledge_base = knowledge_base
    self._mount_path = None
    self._storage_writer = storage_writer
    self._text_prepend = None

    self.number_of_produced_analysis_reports = 0
    self.number_of_produced_event_tags = 0

  @property
  def abort(self):
    """bool: True if the analysis should be aborted."""
    return self._abort

  @property
  def data_location(self):
    """str: path to the data files."""
    return self._data_location

  @property
  def platform(self):
    """str: platform."""
    return self._knowledge_base.platform

  def GetDisplayNameForPathSpec(self, path_spec):
    """Retrieves the display name for a path specification.

    Args:
      path_spec (dfvfs.PathSpec): path specification.

    Returns:
      str: human readable version of the path specification.
    """
    return path_helper.PathHelper.GetDisplayNameForPathSpec(
        path_spec, mount_path=self._mount_path, text_prepend=self._text_prepend)

  def GetUsernameForPath(self, path):
    """Retrieves a username for a specific path.

    This is determining if a specific path is within a user's directory and
    returning the username of the user if so.

    Args:
      path (str): path.

    Returns:
      str: username or None if the path does not appear to be within a user's
          directory.
    """
    return self._knowledge_base.GetUsernameForPath(path)

  def ProduceAnalysisReport(self, plugin):
    """Produces an analysis report.

    Args:
      plugin (AnalysisPlugin): plugin.
    """
    analysis_report = plugin.CompileReport(self)
    if not analysis_report:
      return

    analysis_report.time_compiled = timelib.Timestamp.GetNow()

    plugin_name = getattr(analysis_report, u'plugin_name', plugin.plugin_name)
    if plugin_name:
      analysis_report.plugin_name = plugin_name

    if self._event_filter_expression:
      # TODO: rename filter string when refactoring the analysis reports.
      analysis_report.filter_string = self._event_filter_expression

    self._storage_writer.AddAnalysisReport(analysis_report)

    self.number_of_produced_analysis_reports += 1
    self.number_of_produced_event_tags = (
        self._storage_writer.number_of_event_tags)

  def ProduceEventTag(self, event_tag):
    """Produces an event tag.

    Args:
      event_tag (EventTag): event tag.
    """
    self._storage_writer.AddEventTag(event_tag)

    self.number_of_produced_event_tags += 1

  def SignalAbort(self):
    """Signals the analysis plugins to abort."""
    self._abort = True



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the pinfo CLI tool."""

import json
import unittest

from plaso.cli import views as cli_views
from plaso.cli import pinfo_tool
from plaso.lib import errors
from plaso.lib import timelib

from tests.cli import test_lib


class PinfoToolTest(test_lib.CLIToolTestCase):
  """Tests for the pinfo CLI tool."""

  # TODO: add test for _CalculateStorageCounters.
  # TODO: add test for _CompareStorages.
  # TODO: add test for _PrintAnalysisReportCounter.
  # TODO: add test for _PrintAnalysisReportsDetails.
  # TODO: add test for _PrintErrorsDetails.
  # TODO: add test for _PrintEventLabelsCounter.
  # TODO: add test for _PrintParsersCounter.
  # TODO: add test for _PrintPreprocessingInformation.
  # TODO: add test for _PrintSessionsDetails.
  # TODO: add test for _PrintSessionsOverview.
  # TODO: add test for _PrintTasksInformation.
  # TODO: add test for _PrintStorageInformationAsText.

  def testCompareStorages(self):
    """Tests the CompareStorages function."""
    output_writer = test_lib.TestOutputWriter(encoding=u'utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)

    test_file1 = self._GetTestFilePath([u'psort_test.json.plaso'])
    test_file2 = self._GetTestFilePath([u'pinfo_test.json.plaso'])

    options = test_lib.TestOptions()
    options.compare_storage_file = test_file1
    options.storage_file = test_file1

    test_tool.ParseOptions(options)

    self.assertTrue(test_tool.CompareStorages())

    output = output_writer.ReadOutput()
    self.assertEqual(output, b'Storage files are identical.\n')

    options = test_lib.TestOptions()
    options.compare_storage_file = test_file1
    options.storage_file = test_file2

    test_tool.ParseOptions(options)

    self.assertFalse(test_tool.CompareStorages())

    output = output_writer.ReadOutput()
    self.assertEqual(output, b'Storage files are different.\n')

  def testParseArguments(self):
    """Tests the ParseArguments function."""
    output_writer = test_lib.TestOutputWriter(encoding=u'utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)

    result = test_tool.ParseArguments()
    self.assertFalse(result)

    # TODO: check output.
    # TODO: improve test coverage.

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    output_writer = test_lib.TestOutputWriter(encoding=u'utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.storage_file = self._GetTestFilePath([u'pinfo_test.json.plaso'])

    test_tool.ParseOptions(options)

    options = test_lib.TestOptions()

    with self.assertRaises(errors.BadConfigOption):
      test_tool.ParseOptions(options)

    # TODO: improve test coverage.

  def testPrintStorageInformationAsText(self):
    """Tests the _PrintStorageInformationAsText function."""
    output_writer = test_lib.TestOutputWriter(encoding=u'utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)

    test_filename = u'pinfo_test.json.plaso'
    format_version = u'20170121'
    plaso_version = u'20170714'
    session_identifier = u'81ca8bf6-285d-4bac-bb7c-d339f07d7728'
    session_start_time = u'2017-07-15T07:55:31.382324+00:00'
    session_completion_time = u'2017-07-15T07:55:32.171567+00:00'

    command_line_arguments = (
        u'./tools/log2timeline.py --partition=all --quiet '
        u'pinfo_test.json.plaso test_data/tsk_volume_system.raw')

    enabled_parser_names = u', '.join([
        u'android_app_usage', u'asl_log', u'bash', u'bencode',
        u'bencode/bencode_transmission', u'bencode/bencode_utorrent',
        u'binary_cookies', u'bsm_log', u'chrome_cache', u'chrome_preferences',
        u'cups_ipp', u'custom_destinations', u'dockerjson', u'dpkg', u'esedb',
        u'esedb/esedb_file_history', u'esedb/msie_webcache', u'filestat',
        u'firefox_cache', u'firefox_cache2', u'hachoir', u'java_idx', u'lnk',
        u'mac_appfirewall_log', u'mac_keychain', u'mac_securityd', u'mactime',
        u'macwifi', u'mcafee_protection', u'mft', u'msiecf', u'olecf',
        u'olecf/olecf_automatic_destinations', u'olecf/olecf_default',
        u'olecf/olecf_document_summary', u'olecf/olecf_summary', u'openxml',
        u'opera_global', u'opera_typed_history', u'pe', u'plist',
        u'plist/airport', u'plist/apple_id', u'plist/ipod_device',
        u'plist/macosx_bluetooth', u'plist/macosx_install_history',
        u'plist/macuser', u'plist/maxos_software_update',
        u'plist/plist_default', u'plist/safari_history', u'plist/spotlight',
        u'plist/spotlight_volume', u'plist/time_machine', u'pls_recall',
        u'popularity_contest', u'prefetch', u'recycle_bin',
        u'recycle_bin_info2', u'rplog', u'sccm', u'selinux', u'skydrive_log',
        u'skydrive_log_old', u'sqlite', u'sqlite/android_calls',
        u'sqlite/android_sms', u'sqlite/android_webview',
        u'sqlite/android_webviewcache', u'sqlite/appusage',
        u'sqlite/chrome_cookies', u'sqlite/chrome_extension_activity',
        u'sqlite/chrome_history', u'sqlite/firefox_cookies',
        u'sqlite/firefox_downloads', u'sqlite/firefox_history',
        u'sqlite/google_drive', u'sqlite/imessage',
        u'sqlite/kik_messenger', u'sqlite/ls_quarantine',
        u'sqlite/mac_document_versions', u'sqlite/mackeeper_cache',
        u'sqlite/skype', u'sqlite/twitter_ios', u'sqlite/zeitgeist',
        u'symantec_scanlog', u'syslog', u'syslog/cron', u'syslog/ssh',
        u'systemd_journal', u'usnjrnl', u'utmp', u'utmpx', u'winevt',
        u'winevtx', u'winfirewall', u'winiis', u'winjob', u'winreg',
        u'winreg/appcompatcache', u'winreg/bagmru', u'winreg/ccleaner',
        u'winreg/explorer_mountpoints2', u'winreg/explorer_programscache',
        u'winreg/microsoft_office_mru', u'winreg/microsoft_outlook_mru',
        u'winreg/mrulist_shell_item_list', u'winreg/mrulist_string',
        u'winreg/mrulistex_shell_item_list', u'winreg/mrulistex_string',
        u'winreg/mrulistex_string_and_shell_item',
        u'winreg/mrulistex_string_and_shell_item_list', u'winreg/msie_zone',
        u'winreg/mstsc_rdp', u'winreg/mstsc_rdp_mru', u'winreg/network_drives',
        u'winreg/userassist', u'winreg/windows_boot_execute',
        u'winreg/windows_boot_verify', u'winreg/windows_run',
        u'winreg/windows_sam_users', u'winreg/windows_services',
        u'winreg/windows_shutdown', u'winreg/windows_task_cache',
        u'winreg/windows_timezone', u'winreg/windows_typed_urls',
        u'winreg/windows_usb_devices', u'winreg/windows_usbstor_devices',
        u'winreg/windows_version', u'winreg/winlogon', u'winreg/winrar_mru',
        u'winreg/winreg_default', u'xchatlog', u'xchatscrollback',
        u'zsh_extended_history'])

    table_view = cli_views.ViewsFactory.GetTableView(
        cli_views.ViewsFactory.FORMAT_TYPE_CLI,
        title=u'Plaso Storage Information')
    table_view.AddRow([u'Filename', test_filename])
    table_view.AddRow([u'Format version', format_version])
    table_view.AddRow([u'Serialization format', u'json'])
    table_view.Write(output_writer)

    table_view = cli_views.ViewsFactory.GetTableView(
        cli_views.ViewsFactory.FORMAT_TYPE_CLI, title=u'Sessions')
    table_view.AddRow([session_identifier, session_start_time])
    table_view.Write(output_writer)

    title = u'Session: {0!s}'.format(session_identifier)
    table_view = cli_views.ViewsFactory.GetTableView(
        cli_views.ViewsFactory.FORMAT_TYPE_CLI, title=title)
    table_view.AddRow([u'Start time', session_start_time])
    table_view.AddRow([u'Completion time', session_completion_time])
    table_view.AddRow([u'Product name', u'plaso'])
    table_view.AddRow([u'Product version', plaso_version])
    table_view.AddRow([u'Command line arguments', command_line_arguments])
    table_view.AddRow([u'Parser filter expression', u'N/A'])
    table_view.AddRow([u'Enabled parser and plugins', enabled_parser_names])
    table_view.AddRow([u'Preferred encoding', u'UTF-8'])
    table_view.AddRow([u'Debug mode', u'False'])
    table_view.AddRow([u'Filter file', u'N/A'])
    table_view.Write(output_writer)

    table_view = cli_views.ViewsFactory.GetTableView(
        cli_views.ViewsFactory.FORMAT_TYPE_CLI,
        column_names=[u'Parser (plugin) name', u'Number of events'],
        title=u'Events generated per parser')
    table_view.AddRow([u'filestat', u'3'])
    table_view.AddRow([u'Total', u'3'])
    table_view.Write(output_writer)

    expected_output = output_writer.ReadOutput()

    expected_output = (
        b'{0:s}'
        b'No errors stored.\n'
        b'\n'
        b'No analysis reports stored.\n'
        b'\n').format(expected_output)

    test_file = self._GetTestFilePath([test_filename])

    options = test_lib.TestOptions()
    options.storage_file = test_file
    options.output_format = u'text'

    test_tool.ParseOptions(options)

    test_tool.PrintStorageInformation()

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split(b'\n'), expected_output.split(b'\n'))

  def testPrintStorageInformationAsJSON(self):
    """Tests the _PrintStorageInformationAsJSON function."""
    test_filename = u'pinfo_test.json.plaso'
    session_identifier = u'81ca8bf6285d4bacbb7cd339f07d7728'
    session_start_time = timelib.Timestamp.CopyFromString(
        u'2017-07-15 07:55:31.382324')
    output_writer = test_lib.TestOutputWriter(encoding=u'utf-8')
    test_tool = pinfo_tool.PinfoTool(output_writer=output_writer)
    test_file = self._GetTestFilePath([test_filename])

    options = test_lib.TestOptions()
    options.storage_file = test_file
    options.output_format = u'json'

    test_tool.ParseOptions(options)

    test_tool.PrintStorageInformation()
    output = output_writer.ReadOutput()
    json_output = json.loads(output)

    first_session_identifier = u'session_{0:s}'.format(session_identifier)
    first_session = json_output.get(first_session_identifier, None)
    self.assertIsNotNone(first_session)

    self.assertEqual(first_session[u'identifier'], session_identifier)
    self.assertEqual(first_session[u'start_time'], session_start_time)

    parsers_counter = first_session[u'parsers_counter']
    self.assertEqual(parsers_counter[u'total'], 3)
    self.assertEqual(parsers_counter[u'filestat'], 3)


if __name__ == '__main__':
  unittest.main()
